# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0010_auto_20150730_1112'),
    ]

    operations = [
        migrations.AlterField(
            model_name='account',
            name='api_token',
            field=models.CharField(help_text='From https://pinboard.in/settings/password eg, "philgyford:1234567890ABCDEFGHIJ"', max_length=51),
        ),
        migrations.AlterField(
            model_name='account',
            name='time_created',
            field=models.DateTimeField(help_text='The time this item was created in the database.', auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='time_modified',
            field=models.DateTimeField(help_text='The time this item was last saved to the database.', auto_now=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='url',
            field=models.URLField(unique=True, help_text="eg, 'https://pinboard.in/u:philgyford'", max_length=255),
        ),
        migrations.AlterField(
            model_name='account',
            name='username',
            field=models.CharField(unique=True, help_text="eg, 'philgyford'", max_length=30),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='description',
            field=models.TextField(help_text="The 'extended' text description.", blank=True),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='fetch_time',
            field=models.DateTimeField(help_text="The time the item's data was last fetched, and was new or changed.", null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='is_private',
            field=models.BooleanField(help_text='If set, this item will not be shown on public-facing pages.', default=False),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='permalink',
            field=models.URLField(help_text="URL of the item on the service's website.", blank=True),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='post_time',
            field=models.DateTimeField(help_text='The time this was created on Pinboard.', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='raw',
            field=models.TextField(help_text='eg, the raw JSON from the API.', blank=True),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='summary',
            field=models.CharField(help_text="eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc. No HTML.", blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='time_created',
            field=models.DateTimeField(help_text='The time this item was created in the database.', auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='time_modified',
            field=models.DateTimeField(help_text='The time this item was last saved to the database.', auto_now=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import taggit.managers


class Migration(migrations.Migration):

    dependencies = [
        ('taggit', '0001_initial'),
        ('pinboard', '0007_auto_20150724_1957'),
    ]

    operations = [
        migrations.AddField(
            model_name='bookmark',
            name='tags',
            field=taggit.managers.TaggableManager(to='taggit.Tag', through='taggit.TaggedItem', help_text='A comma-separated list of tags.', verbose_name='Tags'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-03 16:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0012_auto_20160503_1457'),
    ]

    operations = [
        migrations.AddField(
            model_name='photoset',
            name='photos_raw',
            field=models.TextField(blank=True, help_text='The raw JSON from the API listing the photos.'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0029_auto_20151110_1308'),
    ]

    operations = [
        migrations.AlterField(
            model_name='account',
            name='consumer_key',
            field=models.CharField(help_text='(API Key) From https://apps.twitter.com', blank=True, max_length=255),
        ),
    ]



import datetime
import pytz

from django import template
from django.utils.html import format_html

from ..models import Photo, Photoset, User
from ...core.templatetags.ditto_core import display_time
from ...core.utils import get_annual_item_counts


register = template.Library()


@register.simple_tag
def recent_photos(nsid=None, limit=10):
    """Returns a QuerySet of recent public Photos, in reverse-chronological
    order.

    Keyword arguments:
    nsid -- A Flickr user's NSID. If not supplied, we fetch
                    Photos for all Flickr users that have Accounts.
    limit -- Maximum number to fetch. Default is 10.
    """
    photos = Photo.public_photo_objects.all()
    if nsid is not None:
        photos = photos.filter(user__nsid=nsid)
    photos = photos.prefetch_related('user')
    return photos[:limit]

@register.simple_tag
def day_photos(date, nsid=None, time='post_time'):
    """Returns a QuerySet of public Photos posted on a specific date.

    Arguments:
    date -- A date object.

    Keyword arguments:
    nsid -- A Flickr user's NSID. If not supplied, we fetch
                    Photos for all Flickr users that have Accounts.
    time -- A string, either 'post_time' (default) or 'taken_time'.
    """
    if time not in ['post_time', 'taken_time']:
        raise ValueError("`time` must be either 'post_time' or "
                        "'taken_time', not '%s'." % time)

    start = datetime.datetime.combine(date, datetime.time.min).replace(
                                                            tzinfo=pytz.utc)
    end   = datetime.datetime.combine(date, datetime.time.max).replace(
                                                            tzinfo=pytz.utc)
    photos = Photo.public_photo_objects

    if time == 'taken_time':
        photos = photos.filter(taken_time__range=[start, end])
    else:
        photos = photos.filter(post_time__range=[start, end])

    if nsid is not None:
        photos = photos.filter(user__nsid=nsid)

    photos = photos.prefetch_related('user')
    return photos

@register.simple_tag
def photosets(nsid=None, limit=10):
    """Returns a QuerySet of recent Photosets.

    Keyword arguments:
    nsid -- A Flickr user's NSID. If not supplied, we fetch
                    Photosets for all Flickr users that have Accounts.
    limit -- Maximum number to fetch. Default is 10.
    """
    photosets = Photoset.objects.all()
    if nsid is not None:
        photosets = photosets.filter(user__nsid=nsid)
    return photosets.prefetch_related('primary_photo', 'user')[:limit]


@register.simple_tag
def photo_license(n):
    """Returns the text value of the Photo's license, indicated by the number n.
    Will probably be an HTML link to more info.
    """
    licenses = dict((x,y) for x,y in Photo.LICENSES)

    if n in licenses:
        if n in Photo.LICENSE_URLS and Photo.LICENSE_URLS[n] != '':
            return format_html(
                '<a href="%(url)s" title="More about permissions">%(name)s</a>' % {
                    'url': Photo.LICENSE_URLS[n],
                    'name': licenses[n],
                })
        else:
            return licenses[n]
    else:
        return '[missing]'


@register.simple_tag
def annual_photo_counts(nsid=None, count_by='post_time'):
    """
    Get the number of public Photos per year.
    Returns a list of dicts, sorted by year, like:
        [ {'year': 2015, 'count': 1234}, {'year': 2016, 'count': 9876} ]

    Keyword arguments:
    nsid -- A Flickr user's NSID or None (for Photos by all Users).
    count_by -- A string, either 'post_time' (default) or 'taken_time'.
    """

    if count_by not in ['post_time', 'taken_time']:
        raise ValueError("`count_by` must be either 'post_time' or "
                        "'taken_time', not '%s'." % count_by)

    qs = Photo.public_photo_objects

    if nsid is not None:
        qs = qs.filter(user__nsid=nsid)

    if count_by == 'taken_time':
        field_name = 'taken_year'
    else:
        field_name = 'post_year'

    return get_annual_item_counts(qs, field_name)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0006_auto_20150730_1450'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='tweet',
            name='in_reply_to_status_id_str',
        ),
        migrations.RemoveField(
            model_name='tweet',
            name='in_reply_to_user_id_str',
        ),
        migrations.RemoveField(
            model_name='tweet',
            name='quoted_status_id_str',
        ),
        migrations.RemoveField(
            model_name='tweet',
            name='twitter_id_str',
        ),
        migrations.AlterField(
            model_name='tweet',
            name='favorite_count',
            field=models.PositiveIntegerField(default=0, help_text=b'Approximately how many times this had been favorited when fetched'),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='retweet_count',
            field=models.PositiveIntegerField(default=0, help_text=b'Number of times this had been retweeted when fetched'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-07-13 11:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0048_auto_20160615_1045'),
    ]

    operations = [
        migrations.AlterField(
            model_name='media',
            name='aspect_ratio',
            field=models.CharField(blank=True, help_text='eg, "4:3" or "16:9"', max_length=21),
        ),
        migrations.AlterField(
            model_name='media',
            name='dash_url',
            field=models.URLField(blank=True, verbose_name='MPEG-DASH URL (.mpd, streaming)'),
        ),
        migrations.AlterField(
            model_name='media',
            name='xmpeg_url',
            field=models.URLField(blank=True, verbose_name='X-MPEG URL (HLS, .m3u8, streaming'),
        ),
    ]



from imagekit import ImageSpec, register
from imagekit.processors import Adjust, ResizeToFill, ResizeToFit, Transpose


# Info about different Flickr image sizes:
# https://www.flickr.com/services/api/misc.urls.html
#
# Also see ditto.flickr.models.Photo


# BASE CLASSES

class FlickrSpec(ImageSpec):
    "Base class for most specs. Keeps same proportions."
    format = 'JPEG'
    options = {'quality': 80}
    # If original image is smaller, don't enlarge it:
    upscale = False

    def __init__(self, source):
        self.processors = [
                Transpose(),
                ResizeToFit(self.width, self.height, upscale=self.upscale),
                Adjust(sharpness=2.0),]
        super().__init__(source)


class FlickrSquareSpec(ImageSpec):
    "Base class for the square specs. Crops to a square."
    format = 'JPEG'
    options = {'quality': 70}
    # If original image is smaller, enlarge it to fill:
    upscale = True

    def __init__(self, source):
        self.processors = [
                Transpose(),
                ResizeToFill(self.size, self.size, upscale=self.upscale),
                Adjust(sharpness=2.0),]
        super().__init__(source)


# SQUARES

class Square(FlickrSquareSpec):
    size = 75
register.generator('ditto_flickr:square', Square)

class LargeSquare(FlickrSquareSpec):
    size = 150
register.generator('ditto_flickr:large_square', LargeSquare)


# NATURAL PROPORTIONS

class Thumbnail(FlickrSpec):
    width = 100
    height = 100
    # If original image is smaller, enlarge it to fit:
    upscale = True
register.generator('ditto_flickr:thumbnail', Thumbnail)

class Small(FlickrSpec):
    width = 240
    height = 240
register.generator('ditto_flickr:small', Small)

class Small320(FlickrSpec):
    width = 320
    height = 320
register.generator('ditto_flickr:small_320', Small320)

class Medium(FlickrSpec):
    width = 500
    height = 500
register.generator('ditto_flickr:medium', Medium)

class Medium640(FlickrSpec):
    width = 640
    height = 640
register.generator('ditto_flickr:medium_640', Medium640)

class Medium800(FlickrSpec):
    width = 800
    height = 800
register.generator('ditto_flickr:medium_800', Medium800)

class Large(FlickrSpec):
    width = 1024
    height = 1024
register.generator('ditto_flickr:large', Large)

class Large1600(FlickrSpec):
    width = 1600
    height = 1600
register.generator('ditto_flickr:large_1600', Large1600)

class Large2048(FlickrSpec):
    width = 2048
    height = 2048
register.generator('ditto_flickr:large_2048', Large2048)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0018_tweet_text_html'),
    ]

    operations = [
        migrations.CreateModel(
            name='Photo',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('twitter_id', models.BigIntegerField(unique=True)),
                ('media_url', models.URLField(help_text='URL of the image itself')),
                ('page_url', models.URLField(help_text='URL of its page on Twitter.com')),
                ('is_private', models.BooleanField(default=False, help_text='If true, this item will not be shown on public-facing pages.')),
                ('large_w', models.PositiveSmallIntegerField(null=True, verbose_name='Large width', blank=True)),
                ('large_h', models.PositiveSmallIntegerField(null=True, verbose_name='Large height', blank=True)),
                ('medium_w', models.PositiveSmallIntegerField(null=True, verbose_name='Medium width', blank=True)),
                ('medium_h', models.PositiveSmallIntegerField(null=True, verbose_name='Medium height', blank=True)),
                ('small_w', models.PositiveSmallIntegerField(null=True, verbose_name='Small width', blank=True)),
                ('small_h', models.PositiveSmallIntegerField(null=True, verbose_name='Small height', blank=True)),
                ('thumb_w', models.PositiveSmallIntegerField(null=True, verbose_name='Thumbnail width', blank=True)),
                ('thumb_h', models.PositiveSmallIntegerField(null=True, verbose_name='Thumbnail height', blank=True)),
            ],
            options={
                'ordering': ['time_created'],
            },
        ),
        migrations.AddField(
            model_name='tweet',
            name='photos_count',
            field=models.PositiveSmallIntegerField(default=0, help_text='Number of Photos attached to this Tweet', blank=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='is_private',
            field=models.BooleanField(default=False, help_text='If true, this item will not be shown on public-facing pages.'),
        ),
        migrations.AddField(
            model_name='photo',
            name='tweet',
            field=models.ForeignKey(to='twitter.Tweet', on_delete=models.CASCADE),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-25 10:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0017_auto_20160524_1350'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='photos_url',
            field=models.URLField(max_length=255, verbose_name='Photos URL at Flickr'),
        ),
        migrations.AlterField(
            model_name='user',
            name='profile_url',
            field=models.URLField(max_length=255, verbose_name='Avatar URL on Flickr'),
        ),
    ]



# coding: utf-8
import argparse
import os

from django.core.management.base import BaseCommand, CommandError

from ...ingest import TweetIngester


class Command(BaseCommand):
    """Imports tweets from a downloaded archive of Tweets.
    Request your archive from https://twitter.com/settings/account

    Usage:
    ./manage.py import_tweets --path=/Users/phil/Downloads/12552_dbeb4be9b8ff5f76d7d486c005cc21c9faa61f66
    """

    help = "Imports a complete history of tweets from a downloaded archive"

    def add_arguments(self, parser):
        parser.add_argument(
            '--path',
            action='store',
            default=False,
            help='Path to the directory that is the archive'
        )

    def handle(self, *args, **options):
        # Location of the directory holding the tweet JSON files within the
        # archive:
        subpath = '/data/js/tweets'

        if options['path']:
            if os.path.isdir(options['path']):
                tweets_dir = '%s%s' % (options['path'], subpath)
                if os.path.isdir(tweets_dir):
                    result = TweetIngester().ingest(directory=tweets_dir)
                else:
                    raise CommandError("Expected to find a directory at '%s' containing JSON files" % tweets_dir)
            else:
                raise CommandError("Can't find a directory at '%s'" % options['path'])
        else:
            raise CommandError("Specify the location of the archive, eg --path=/Path/To/1234567890_abcdefg12345")

        if options.get('verbosity', 1) > 0:
            if result['success']:
                tweetnoun = 'tweet' if result['tweets'] == 1 else 'tweets'
                filenoun = 'file' if result['files'] == 1 else 'files'

                self.stdout.write(
                    'Imported %s %s from %s %s' % (
                    result['tweets'], tweetnoun, result['files'], filenoun))
            else:

                self.stderr.write('Failed to import tweets: %s' % (
                                                        result['messages'][0]))




from django.db import models

from taggit.managers import _TaggableManager

from ..core.managers import PublicItemManager


class PhotosManager(models.Manager):
    """Returns public AND PRIVATE Photos posted by one of the Users with
    Accounts here.
    As opposed to just Photos, which includes Photos by any User that
    have been favorited by a User with an Account.
    """
    def get_queryset(self):
        from .models import User
        users = User.objects_with_accounts.all()
        return super().get_queryset().filter(user__in=users)


class PublicPhotosManager(PublicItemManager):
    """Returns public Photos posted by one of the Users with Accounts here.
    As opposed to just public Photos, which includes Photos by any User that
    have been favorited by a User with an Account.
    """
    def get_queryset(self):
        from .models import User
        users = User.objects_with_accounts.all()
        return super().get_queryset().filter(user__in=users)


class WithAccountsManager(models.Manager):
    "Returns Flickr Users who have an Account on Ditto."

    def get_queryset(self):
        from .models import Account
        accounts = Account.objects.exclude(user__isnull=True)
        user_ids = [account.user.id for account in accounts]
        return super().get_queryset().filter(pk__in=user_ids)


class _PhotoTaggableManager(_TaggableManager):
    """Providing some extra features related to private Photos."""

    def most_common(self):
        """Gets the most commonly-used tags but:
            * Doesn't count tags on private Photos
        Overriding django-taggit's standard `most_common()` method.
        """
        extra_filters = {
            'photo__is_private': False,
        }

        return self.get_queryset(extra_filters).annotate(
            num_times=models.Count(self.through.tag_relname())
        ).order_by('-num_times')




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0026_auto_20151110_1131'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user',
            name='profile_image_url',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-11-13 10:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0053_tweet_post_year'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='name',
            field=models.CharField(help_text="eg, 'Samuel Pepys'", max_length=50),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.core.validators


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('time_created', models.DateTimeField(auto_now_add=True)),
                ('time_modified', models.DateTimeField(auto_now=True)),
                ('username', models.CharField(help_text=b"eg, 'philgyford'", unique=True, max_length=30)),
                ('url', models.URLField(help_text=b"eg, 'https://pinboard.in/u:philgyford'", max_length=255)),
                ('api_token', models.CharField(help_text=b'From https://pinboard.in/settings/password eg, "philgyford:1234567890ABCDEFGHIJ"', max_length=51)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Bookmark',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('time_created', models.DateTimeField(auto_now_add=True)),
                ('time_modified', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=255, blank=True)),
                ('permalink', models.URLField(help_text=b"URL of the item on the service's website.", blank=True)),
                ('summary', models.TextField(help_text=b"eg, First paragraph of a blog post, start of the description of a photo, all of a Tweet's text, etc.", blank=True)),
                ('is_private', models.BooleanField(default=False, help_text=b'If True, this item should NOT be shown on public-facing pages.')),
                ('fetch_time', models.DateTimeField(help_text=b'The time the Raw data was last fetched.', null=True, blank=True)),
                ('raw', models.TextField(help_text=b'The raw JSON from the API.', blank=True)),
                ('url', models.TextField(unique=True, validators=[django.core.validators.URLValidator()])),
                ('post_time', models.DateTimeField()),
                ('description', models.TextField(help_text=b"The 'extended' text description.", blank=True)),
                ('to_read', models.BooleanField(default=False)),
                ('shared', models.BooleanField(default=True)),
                ('account', models.ForeignKey(to='pinboard.Account', on_delete=models.CASCADE)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-15 10:38
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0045_media_original_image_file'),
    ]

    operations = [
        migrations.RenameField(
            model_name='media',
            old_name='original_image_file',
            new_name='image_file',
        ),
        migrations.AlterField(
            model_name='media',
            name='image_url',
            field=models.URLField(help_text='URL of the image itself on Twitter.com'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-29 15:59
from __future__ import unicode_literals

import ditto.core.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0007_auto_20160414_1637'),
    ]

    operations = [
        migrations.CreateModel(
            name='Photoset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('flickr_id', models.BigIntegerField(unique=True)),
                ('title', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, help_text='Can contain HTML')),
                ('photo_count', models.PositiveIntegerField(default=0)),
                ('video_count', models.PositiveIntegerField(default=0)),
                ('view_count', models.PositiveIntegerField(default=0, help_text='How many times this had been viewed when fetched')),
                ('comment_count', models.PositiveIntegerField(default=0, help_text='How many comments this had when fetched')),
                ('last_update_time', models.DateTimeField(blank=True, help_text='The last time the set was modified on Flickr. UTC.', null=True)),
                ('flickr_created_time', models.DateTimeField(blank=True, help_text='When the set was created on Flickr. UTC.', null=True)),
                ('raw', models.TextField(blank=True, help_text='The raw JSON from the API.')),
            ],
            options={
                'ordering': ['-flickr_created_time'],
            },
            bases=(ditto.core.models.DiffModelMixin, models.Model),
        ),
        migrations.AlterField(
            model_name='photo',
            name='comment_count',
            field=models.PositiveIntegerField(default=0, help_text='How many comments this had when fetched'),
        ),
        migrations.AlterField(
            model_name='photo',
            name='sizes_raw',
            field=models.TextField(blank=True, help_text='The raw JSON from the API - flickr.photos.getSizes.'),
        ),
        migrations.AlterField(
            model_name='user',
            name='raw',
            field=models.TextField(blank=True, help_text='The raw JSON from the API.'),
        ),
        migrations.AddField(
            model_name='photoset',
            name='primary_photo',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='flickr.Photo'),
        ),
        migrations.AddField(
            model_name='photoset',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='flickr.User'),
        ),
    ]



from django.http import Http404
from django.utils.translation import ugettext as _
from django.views.generic import DetailView, ListView
from django.views.generic.detail import SingleObjectMixin

from ..core.views import PaginatedListView
from .models import Account, Bookmark, BookmarkTag


class SingleAccountMixin(SingleObjectMixin):
    "For views which list bookmarks and also need an Account object."
    slug_field = 'username'
    slug_url_kwarg = 'username'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=Account.objects.all())
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['account'] = self.object
        return context


class HomeView(PaginatedListView):
    "List all recent Bookmarks and all Accounts"
    template_name = 'pinboard/home.html'
    queryset = Bookmark.public_objects.all().prefetch_related('account')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['account_list'] = Account.objects.all()
        return context


class ToreadListView(PaginatedListView):
    template_name = 'pinboard/toread_list.html'
    queryset = Bookmark.public_toread_objects.all().prefetch_related('account')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['account_list'] = Account.objects.all()
        return context


class AccountDetailView(SingleAccountMixin, PaginatedListView):
    "A single Pinboard Account and its Bookmarks."
    template_name = 'pinboard/account_detail.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['bookmark_list'] = context['object_list']
        return context

    def get_queryset(self):
        "Show all the public Bookmarks associated with this account."
        return Bookmark.public_objects.filter(account=self.object)\
                                                .prefetch_related('account')


class AccountToreadView(SingleAccountMixin, PaginatedListView):
    "A single Pinboard Account and its 'to read' Bookmarks."
    template_name = 'pinboard/account_toread.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['bookmark_list'] = context['object_list']
        return context

    def get_queryset(self):
        "Show all the public Bookmarks associated with this account."
        return Bookmark.public_toread_objects.filter(account=self.object)\
                                                .prefetch_related('account')


class BookmarkDetailView(DetailView):
    "A single Bookmark, from one Account"
    model = Bookmark
    # Only display public bookmarks; private ones will 404.
    queryset = Bookmark.public_objects.all()
    slug_field = 'url_hash'
    slug_url_kwarg = 'hash'


class TagListView(ListView):
    template_name = 'pinboard/tag_list.html'
    context_object_name = 'tag_list'

    def get_queryset(self):
        return Bookmark.tags.most_common()[:100]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['account_list'] = Account.objects.all()
        return context


class TagDetailView(SingleObjectMixin, PaginatedListView):
    "All Bookmarks with a certain tag from all Accounts"
    template_name = 'pinboard/tag_detail.html'
    allow_empty = False

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=BookmarkTag.objects.all())
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['tag'] = self.object
        context['account_list'] = Account.objects.all()
        context['bookmark_list'] = context['object_list']
        return context

    def get_queryset(self):
        "Show all the public Bookmarks associated with this tag."
        return Bookmark.public_objects\
                                .filter(tags__slug__in=[self.object.slug])\
                                .prefetch_related('account')


class AccountTagDetailView(SingleAccountMixin, PaginatedListView):
    "All Bookmarks with a certain Tag from one Account"
    template_name = 'pinboard/account_tag_detail.html'
    allow_empty = False

    def get(self, request, *args, **kwargs):
        self.tag_object = self.get_tag_object()
        return super().get(request, *args, **kwargs)

    def get_tag_object(self):
        """Custom method for fetching the Tag."""
        try:
            obj = BookmarkTag.objects.get(slug=self.kwargs['tag_slug'])
        except BookmarkTag.DoesNotExist:
            raise Http404(_("No Tags found matching the query"))
        return obj

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['tag'] = self.tag_object
        context['bookmark_list'] = context['object_list']
        return context

    def get_queryset(self):
        """Show all the public Bookmarks associated with this account."""
        return Bookmark.public_objects.filter(account=self.object,
                                    tags__slug__in=[self.kwargs['tag_slug']])





# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-03 13:06
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0040_auto_20160603_1302'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='media',
            name='tweet',
        ),
    ]



# coding: utf-8
from ....core.management.commands import DittoBaseCommand
from ...fetch.fetchers import FilesFetcher


class Command(DittoBaseCommand):
    """Fetches images and Animated GIFs' video files from Twitter.

    eg:
    ./manage.py fetch_twitter_files
    ./manage.py fetch_twitter_files --all

    """

    help = "Fetches images and Animated GIFs' video files from Twitter"

    singular_noun = 'File'
    plural_noun = 'Files'

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            '--all',
            action='store_true',
            default=False,
            help="Fetch ALL files, even if they've been downloaded before. Otherwise, only fetch files that haven't already been downloaded."
        )

    def handle(self, *args, **options):
        results = FilesFetcher().fetch(fetch_all=options['all'])
        self.output_results(results, options.get('verbosity', 1))




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0003_auto_20160413_0906'),
    ]

    operations = [
        migrations.AddField(
            model_name='photo',
            name='height_mp4_hd',
            field=models.PositiveSmallIntegerField(null=True, verbose_name='HD MP4 height', blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='height_mp4_mobile',
            field=models.PositiveSmallIntegerField(null=True, verbose_name='Mobile MP4 height', blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='height_mp4_site',
            field=models.PositiveSmallIntegerField(null=True, verbose_name='Site MP4 height', blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='height_video_original',
            field=models.PositiveSmallIntegerField(null=True, verbose_name='Original video height', blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='width_mp4_hd',
            field=models.PositiveSmallIntegerField(null=True, verbose_name='HD MP4 width', blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='width_mp4_mobile',
            field=models.PositiveSmallIntegerField(null=True, verbose_name='Mobile MP4 width', blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='width_mp4_site',
            field=models.PositiveSmallIntegerField(null=True, verbose_name='Site MP4 width', blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='width_video_original',
            field=models.PositiveSmallIntegerField(null=True, verbose_name='Original video width', blank=True),
        ),
        migrations.AlterField(
            model_name='photo',
            name='license',
            field=models.CharField(choices=[('0', 'All Rights Reserved'), ('1', 'Attribution-NonCommercial-ShareAlike License'), ('2', 'Attribution-NonCommercial License'), ('3', 'Attribution-NonCommercial-NoDerivs License'), ('4', 'Attribution License'), ('5', 'Attribution-ShareAlike License'), ('6', 'Attribution-NoDerivs License'), ('7', 'No known copyright restrictions'), ('8', 'United States Government Work'), ('9', 'Public Domain Dedication (CC0)'), ('10', 'Public Domain Mark'), ('11', 'Unused'), ('12', 'Unused'), ('13', 'Unused'), ('14', 'Unused'), ('15', 'Unused'), ('16', 'Unused'), ('17', 'Unused'), ('18', 'Unused'), ('19', 'Unused'), ('20', 'Unused')], max_length=50),
        ),
        migrations.AlterField(
            model_name='photo',
            name='taken_granularity',
            field=models.PositiveSmallIntegerField(default=0, choices=[(0, 'Y-m-d H:i:s'), (1, 'Unused'), (2, 'Unused'), (3, 'Unused'), (4, 'Y-m'), (5, 'Unused'), (6, 'Y'), (7, 'Unused'), (8, 'Circa...'), (9, 'Unused'), (10, 'Unused')]),
        ),
    ]



from django.conf.urls import url

from . import views


app_name = 'lastfm'


# The pattern for matching an Album/Artist/Track slug:
slug_chars = '[\w.,:;=@&+%()$!°’~-]+'


urlpatterns = [
    url(
        regex=r"^$",
        view=views.HomeView.as_view(),
        name='home'
    ),
    url(
        regex=r"^library/$",
        view=views.ScrobbleListView.as_view(),
        name='scrobble_list'
    ),
    url(
        regex=r"^library/albums/$",
        view=views.AlbumListView.as_view(),
        name='album_list'
    ),
    url(
        regex=r"^library/artists/$",
        view=views.ArtistListView.as_view(),
        name='artist_list'
    ),
    url(
        regex=r"^library/tracks/$",
        view=views.TrackListView.as_view(),
        name='track_list'
    ),
    url(
        regex=r"^music/(?P<artist_slug>%s)/$" % slug_chars,
        view=views.ArtistDetailView.as_view(),
        name='artist_detail'
    ),
    url(
        regex=r"^music/(?P<artist_slug>%s)/\+albums/$" % slug_chars,
        view=views.ArtistAlbumsView.as_view(),
        name='artist_albums'
    ),
    url(
        regex=r"^music/(?P<artist_slug>%s)/(?P<album_slug>%s)/$" % (
                                                    slug_chars, slug_chars),
        view=views.AlbumDetailView.as_view(),
        name='album_detail'
    ),
    url(
        regex=r"^music/(?P<artist_slug>%s)/_/(?P<track_slug>%s)/$" % (
                                                    slug_chars, slug_chars),
        view=views.TrackDetailView.as_view(),
        name='track_detail'
    ),

    # User pages.

    url(
        regex=r"^user/(?P<username>[a-z0-9]+)/$",
        view=views.UserDetailView.as_view(),
        name='user_detail'
    ),
    url(
        regex=r"^user/(?P<username>[a-z0-9]+)/library/$",
        view=views.UserScrobbleListView.as_view(),
        name='user_scrobble_list'
    ),
    url(
        regex=r"^user/(?P<username>[a-z0-9]+)/library/albums/$",
        view=views.UserAlbumListView.as_view(),
        name='user_album_list'
    ),
    url(
        regex=r"^user/(?P<username>[a-z0-9]+)/library/artists/$",
        view=views.UserArtistListView.as_view(),
        name='user_artist_list'
    ),
    url(
        regex=r"^user/(?P<username>[a-z0-9]+)/library/tracks/$",
        view=views.UserTrackListView.as_view(),
        name='user_track_list'
    ),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0012_auto_20150814_1730'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='favorites',
            field=models.ManyToManyField(related_name='favoriting_users', to='twitter.Tweet'),
        ),
    ]



from imagekit import ImageSpec, register
from imagekit.processors import Adjust, ResizeToFill, ResizeToFit, Transpose


class TwitterSpec(ImageSpec):
    "Base class for Medium and Small specs."
    format = 'JPEG'
    options = {'quality': 80}
    # If original image is smaller, don't enlarge it:
    upscale = False

    def __init__(self, source):
        self.processors = [
                Transpose(),
                ResizeToFit(self.width, self.height, upscale=self.upscale),
                Adjust(sharpness=2.0),]
        super().__init__(source)


# The 'Large' size appears to be the original image, so we're not using
# Imagekit to generate these.
#class Large(TwitterSpec):
    #pass
#
#register.generator('ditto_twitter:large', Large)


class Medium(TwitterSpec):
    width = 1200
    height = 1200

register.generator('ditto_twitter:medium', Medium)


class Small(TwitterSpec):
    width = 680
    height = 680

register.generator('ditto_twitter:small', Small)


class Thumbnail(ImageSpec):
    width = 150
    height = 150
    upscale = True

    def __init__(self, source):
        self.processors = [
                Transpose(),
                ResizeToFill(self.width, self.height, upscale=True),
                Adjust(sharpness=2.0),]
        super().__init__(source)

register.generator('ditto_twitter:thumbnail', Thumbnail)




from django.contrib import admin
from django.db import models
from django.forms import Textarea, TextInput

from taggit.managers import TaggableManager
from taggit.forms import TagWidget

from ..core.admin import DittoItemModelAdmin
from .models import Account, Bookmark


@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = ('username', 'is_active', 'time_created', 'time_modified', )

    fieldsets = (
        (None, {
            'fields': ('username', 'url', 'is_active', )
        }),
        ('API', {
            'fields': ('api_token',),
            'description': 'Your API Token can be found at <a href="https://pinboard.in/settings/password">pinboard.in/settings/password</a>'}),
        ('Data', {
            'fields': ('time_created', 'time_modified',)
        }),
    )
    readonly_fields = ('time_created', 'time_modified',)


@admin.register(Bookmark)
class BookmarkAdmin(DittoItemModelAdmin):
    list_display = ('title', 'post_time', 'account',)
    list_filter = ('post_time', 'is_private', 'to_read', 'account',)

    fieldsets = (
        (None, {
            'fields': ('account', 'title', 'url', 'description', 'summary', 'tags', 'post_time', 'post_year_str', 'url_hash', )
        }),
        ('Options', {
            'fields': ('is_private', 'to_read',)
        }),
        ('Data', {
            'fields': ('raw', 'fetch_time', 'time_created', 'time_modified',)
        }),
    )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {'widget': TextInput(
            attrs={
                'class': 'vLargeTextField',
            }
        )},
        # Reduce the number of rows; similar to Pinboard's description field.
        models.TextField: {'widget': Textarea(
            attrs={
                'class': 'vLargeTextField',
                'cols': 40,
                'rows': 4,
            }
        )},
        # Make the input full-width.
        TaggableManager: {'widget': TagWidget(
            attrs={
                'class': 'vLargeTextField',
            }
        )},
    }

    readonly_fields = ('post_year_str', 'raw', 'fetch_time', 'time_created', 'time_modified',)
    search_fields = ('title', 'url', 'description',)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0003_auto_20150730_1112'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='favorites_count',
            field=models.PositiveIntegerField(default=0, help_text=b'The number of tweets this user has favorited in the account\xe2\x80\x99s lifetime'),
        ),
        migrations.AlterField(
            model_name='user',
            name='followers_count',
            field=models.PositiveIntegerField(default=0, help_text=b'The number of followers this account has'),
        ),
        migrations.AlterField(
            model_name='user',
            name='friends_count',
            field=models.PositiveIntegerField(default=0, help_text=b'Tne number of users this account is following.'),
        ),
        migrations.AlterField(
            model_name='user',
            name='listed_count',
            field=models.PositiveIntegerField(default=0, help_text=b'The number of public lists this user is a member of'),
        ),
        migrations.AlterField(
            model_name='user',
            name='statuses_count',
            field=models.PositiveIntegerField(default=0, help_text=b'The number of tweets, including retweets, by this user'),
        ),
    ]



from django.apps import AppConfig


class DittoPinboardConfig(AppConfig):
    name = 'ditto.pinboard'
    verbose_name = "Ditto Pinboard"

    def ready(self):
        # import ditto.pinboard.signals
        import ditto.pinboard.checks




# coding: utf-8
import datetime
import json
import pytz
import requests
import urllib

from .models import Account, Bookmark
from ..core.utils import datetime_now


PINBOARD_API_ENDPOINT = "https://api.pinboard.in/v1/"
PINBOARD_DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%SZ"
PINBOARD_ALTERNATE_DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
PINBOARD_DATE_FORMAT = "%Y-%m-%d"


class FetchError(Exception):
    pass


class BookmarksFetcher(object):
    """The parent class containing common methods.
    Use one of the child classes to fetch a particular set of Bookmarks.
    """

    def fetch(self):
        raise FetchError('Call a child class like AllBookmarksFetcher or RecentBookmarksFetcher')


    def _fetch(self, fetch_type, params={}, username=None):
        """The main method for making all types of Bookmark requests, and
        saving the data.

        Keyword arguments:
        fetch_type -- 'all', 'date', 'recent' or 'url'.
        params -- Any params specific to the type (eg, url='http://foo.com')
                    These will be used directly with the Pinboard API.
        username -- the username of the one Account to fetch (or None for all).
        """
        # Each element will be a dict, like:
        # {'account':'philgyford', 'success':True, 'fetched':12}
        result = []

        accounts = self._get_accounts(username)

        for account in accounts:
            fetch_time = datetime_now()

            response = self._send_request(fetch_type, params, account)

            if response['success']:
                # Tidy the raw data:
                bookmarks_data = self._parse_response(
                                                fetch_type, response['json'])
                # Create/update in DB:
                self._save_bookmarks(
                                account=account,
                                bookmarks_data=bookmarks_data,
                                fetch_time=fetch_time)
                response['fetched'] = len(bookmarks_data)
                # Don't need to pass this around any more:
                del(response['json'])
            else:
                response['fetched'] = 0

            response['account'] = account.username
            result.append(response)

        return result

    def _get_accounts(self, username):
        """Get all or one active accounts.
        username is None or a username.
        Raises FetchError if we find that none of the required accounts are
        marked as is_active.
        """
        if username is None:
            accounts = Account.objects.filter(is_active=True)
            if len(accounts) == 0:
                raise FetchError("No active accounts were found to fetch.")
        else:
            account = Account.objects.get(username=username)
            if account.is_active:
                accounts = [account]
            else:
                raise FetchError("The account %s is curently marked as inactive." % username)
        return accounts

    def _send_request(self, fetch_type, params, account):
        """Sends a request to the Pinboard API, returns the raw response.

        Returns a dict with a 'success' element: True or False.
        If False, will have a 'messages' element, a list.
        If True, will have a 'json' element with the fetched data in.

        Keyword arguments:
        fetch_type -- 'all', 'date', 'recent' or 'url'.
        params -- Any params needed for this type. eg 'dt':datetime.
        Account -- The account to fetch from.
        """

        url_parts = ['posts']
        if fetch_type in ['date', 'url']:
            url_parts.append('get')
        elif fetch_type == 'recent':
            url_parts.append('recent')
        elif fetch_type == 'all':
            url_parts.append('all')

        url = "{}{}".format(PINBOARD_API_ENDPOINT, "/".join(url_parts))

        params['format'] = "json"
        params['auth_token'] = account.api_token
        query_string = urllib.parse.urlencode(params)

        final_url = "{}?{}".format(url, query_string)

        error_message = ''

        try:
            response = requests.get(final_url)
        except requests.exceptions.ConnectionError as e:
            error_message = "Can't connect to domain."
        except requests.exceptions.Timeout as e:
            error_message = "Connection timed out."
        except requests.exceptions.TooManyRedirects as e:
            error_message = "Too many redirects."
        except requests.exceptions.RequestException as e:
            error_message = "Something went wrong with the request."

        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            # 4xx or 5xx errors:
            error_message = "HTTP Error: %s" % response.status_code
        except NameError:
            if error_message == '':
                error_message = "Something unusual went wrong."

        if error_message:
            return {'account': account.username, 'success': False,
                                                'messages': [error_message,]}
        else:
            return {'account': account.username, 'success': True,
                                                        'json': response.text}


    def _parse_response(self, fetch_type, json_text):
        """Takes the JSON response for a Bookmark from the API and turns it
        into something more pythony.
        Returns a list of bookmark data.

        Keyword arguments:
        fetch_type -- 'all', 'date', 'recent' or 'url'.
        json_text -- The raw JSON returned from the API.
        """

        response = json.loads(json_text)

        # The JSON has different formats depending on what we fetched:
        if fetch_type == 'all':
            posts = response
        else:
            posts = response['posts']

        for bookmark in posts:
            # Before we do anything to it, we give the bookmark a 'json'
            # element with its original state in:
            bookmark['json'] = json.dumps(bookmark)
            # Time string to object:
            bookmark['time'] = datetime.datetime.strptime(
                                        bookmark['time'], '%Y-%m-%dT%H:%M:%SZ'
                                    ).replace(tzinfo=pytz.utc)
            # 'yes'/'no' to booleans:
            bookmark['shared'] = True if bookmark['shared'] == 'yes' else False
            bookmark['toread'] = True if bookmark['toread'] == 'yes' else False
            # String into an array of strings:
            bookmark['tags'] = bookmark['tags'].split()

        return posts

    def _save_bookmarks(self, account, bookmarks_data, fetch_time):
        """Takes the raw JSON response from the API, creates or updates the
        Bookmark objects.

        Keyword arguments:
        account -- The Account object to add these bookmarks for.
        bookmarks_data -- A list, each one data to create a single Bookmark.
        fetch_time -- The UTC time at which these bookmarks were fetched.
        """
        for bookmark in bookmarks_data:
            self._save_bookmark(bookmark, fetch_time, account)

    def _save_bookmark(self, bookmark, fetch_time, account):
        """Takes data for a single bookmark from the API response and creates
        or updates the Bookmark object.

        Keyword arguments:
        bookmark_data -- A list, each one data to create a single Bookmark.
        fetch_time -- The UTC time at which these bookmarks were fetched.
        account -- The Account object to add these bookmarks for.
        """

        bookmark_obj, created = Bookmark.objects.update_or_create(
            account=account,
            url=bookmark['href'],
            defaults={
                'title': bookmark['description'],
                'is_private': not bookmark['shared'],
                'raw': bookmark['json'],
                'description': bookmark['extended'],
                'to_read': bookmark['toread'],
                'fetch_time': fetch_time,
                'post_time': bookmark['time'],
            }
        )

        bookmark_obj.tags.set(*bookmark['tags'])


class AllBookmarksFetcher(BookmarksFetcher):

    def fetch(self, username=None):
        """Fetches all of the Bookmarks for all or one Accounts.
        Creates/updates the Bookmark objects.

        Keyword arguments:
        username -- the username of the one Account to fetch (or None for all).
        """
        return self._fetch(fetch_type='all', username=username)


class DateBookmarksFetcher(BookmarksFetcher):

    def fetch(self, post_date, username=None):
        """Fetches Bookmarks for all or one Accounts on a particular day.
        Creates/updates the Bookmark objects.

        Keyword arguments:
        post_date -- date to fetch in a "YYYY-MM-DD" format string.
        username -- the username of the one Account to fetch (or None for all).

        Raises:
        FetchError if the date format is invalid.
        """
        try:
            dt = datetime.datetime.strptime(post_date, '%Y-%m-%d')
        except ValueError:
            raise FetchError("Invalid date format ('%s')" % post_date)
        else:
            return self._fetch(
                    fetch_type='date', params={'dt': dt}, username=username)


class RecentBookmarksFetcher(BookmarksFetcher):

    def fetch(self, num=10, username=None):
        """Fetches the most recent Bookmarks for all or one Accounts.
        Creates/updates the Bookmark objects.

        Keyword arguments:
        num -- the number of most recent Bookmarks to fetch.
        username -- the username of the one Account to fetch (or None for all).
        """
        return self._fetch(
            fetch_type='recent', params={'count': int(num)}, username=username)


class UrlBookmarksFetcher(BookmarksFetcher):

    def fetch(self, url, username=None):
        """Fetches a single Bookmark (by URL) for all or one Accounts.
        Creates/updates the Bookmark objects.

        Keyword arguments:
        url -- the URL of the Bookmark to fetch.
        username -- the username of the one Account to fetch (or None for all).
        """
        return self._fetch(
                    fetch_type='url', params={'url': url}, username=username)





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0024_auto_20151104_1157'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='media',
            options={'verbose_name': 'Media item', 'ordering': ['time_created'], 'verbose_name_plural': 'Media items'},
        ),
        migrations.AlterField(
            model_name='tweet',
            name='twitter_id',
            field=models.BigIntegerField(db_index=True, unique=True),
        ),
    ]



# coding: utf-8
import argparse

from django.core.management.base import BaseCommand, CommandError

from ...models import Account, Tweet


class Command(BaseCommand):
    """Generates the HTML version of all the Tweets.
    Does this by re-saving every Tweet, one-by-one.

    For one account:
    ./manage.py generate_tweet_html --account=philgyford

    For all accounts:
    ./manage.py generate_tweet_html
    """

    help = "Generates the HTML version of all the Tweets."

    def add_arguments(self, parser):
        parser.add_argument(
            '--account',
            action='store',
            default=False,
            help='Only generate for one Twitter account.',
        )

    def handle(self, *args, **options):
        tweets = Tweet.objects.all()

        # If a screen name is provided, only get the Tweets for that:
        if options['account']:
            screen_name = options['account']
            try:
                Account.objects.get(user__screen_name=screen_name)
            except Account.DoesNotExist:
                raise CommandError("There's no Account with a screen name of '%s'" % screen_name)
            tweets = tweets.filter(user__screen_name=screen_name)

        for tweet in tweets:
            tweet.save()

        if options.get('verbosity', 1) > 0:
            self.stdout.write('Generated HTML for %d Tweets' % tweets.count())




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0014_auto_20150819_1342'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tweet',
            name='favorite_count',
            field=models.PositiveIntegerField(help_text=b'Approximately how many times this had been favorited when fetched', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='retweet_count',
            field=models.PositiveIntegerField(help_text=b'Number of times this had been retweeted when fetched', null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0028_auto_20151110_1139'),
    ]

    # Manually re-jigged to avoid losing any data:
    operations = [
        migrations.AlterField(
            model_name='tweet',
            name='created_at',
            field=models.DateTimeField(help_text='The time the item was originally posted/created on its service.', blank=True, null=True),
        ),
        migrations.RenameField(
            model_name='tweet',
            old_name='created_at',
            new_name='post_time',
        ),
        migrations.AlterModelOptions(
            name='tweet',
            options={'ordering': ['-post_time']},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-15 10:45
from __future__ import unicode_literals

import ditto.twitter.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0047_media_video_file'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='media',
            name='video_file',
        ),
        migrations.AddField(
            model_name='media',
            name='mp4_file',
            field=models.FileField(blank=True, default='', help_text='Only used for Animated GIFs', upload_to=ditto.twitter.models.Media.upload_path),
        ),
    ]



# coding: utf-8
import argparse

from django.core.management.base import CommandError

from . import FetchTwitterCommand
from ...fetch.fetchers import FavoriteTweetsFetcher


class Command(FetchTwitterCommand):
    """Fetches favorites from Twitter.

    Fetch recent tweets favorited by all accounts, since last time:
    ./manage.py fetch_twitter_likes --recent=new

    Fetch 100 most recent tweets favorited by all accounts:
    ./manage.py fetch_twitter_likes --recent=100

    Fetch recent tweets favorited by one account:
    ./manage.py fetch_twitter_likes --recent=new --account=philgyford
    """

    help = "Fetches recent liked Tweets from Twitter"

    recent_help = 'Fetch the most recent liked Tweets, eg "100" or "new".'

    def fetch_tweets(self, screen_name, count):
        return FavoriteTweetsFetcher(screen_name=screen_name).fetch(count=count)




from django.contrib import admin
from django.utils.html import format_html

from ..core.admin import DittoItemModelAdmin
from .models import Account, Album, Artist, Scrobble, Track


@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = ('username', 'realname', 'is_active', 'time_created',)
    search_fields = ('name', 'artist', 'mbid',)

    fieldsets = (
        (None, {
            'fields': ('username', 'realname', 'api_key', 'is_active', )
        }),
        ('Data', {
            'fields': ('time_created', 'time_modified',)
        }),
    )

    readonly_fields = ('time_created', 'time_modified',)


@admin.register(Artist)
class ArtistAdmin(admin.ModelAdmin):
    list_display = ('name', 'mbid', 'time_created',)
    search_fields = ('name', 'mbid',)

    fieldsets = (
        (None, {
            'fields': ('name', 'slug', 'original_slug', 'mbid',)
        }),
        ('Data', {
            'fields': ('time_created', 'time_modified',)
        }),
    )

    readonly_fields = ('time_created', 'time_modified',)


@admin.register(Track)
class TrackAdmin(admin.ModelAdmin):
    list_display = ('name', 'artist', 'mbid', 'time_created',)
    search_fields = ('name', 'artist__name', 'mbid',)

    fieldsets = (
        (None, {
            'fields': ('name', 'slug', 'original_slug', 'artist', 'mbid',)
        }),
        ('Data', {
            'fields': ('time_created', 'time_modified',)
        }),
    )

    readonly_fields = ('time_created', 'time_modified',)
    raw_id_fields = ('artist',)


@admin.register(Album)
class AlbumAdmin(admin.ModelAdmin):
    list_display = ('name', 'artist', 'mbid', 'time_created',)
    search_fields = ('name', 'artist__name',)

    fieldsets = (
        (None, {
            'fields': ('name', 'slug', 'original_slug', 'artist', 'mbid',)
        }),
        ('Data', {
            'fields': ('time_created', 'time_modified',)
        }),
    )

    readonly_fields = ('time_created', 'time_modified',)
    raw_id_fields = ('artist',)


@admin.register(Scrobble)
class ScrobbleAdmin(DittoItemModelAdmin):
    list_display = ('title', 'post_time',)
    list_filter = ('post_time', )
    search_fields = ('title', 'track__name',)

    fieldsets = (
        (None, {
            'fields': ('account', 'artist', 'track', 'album',
                        'post_time', 'post_year_str',)
        }),
        ('Data', {
            'fields': ('raw', 'fetch_time', 'time_created', 'time_modified',)
        }),
    )

    raw_id_fields = ('artist', 'track', 'album',)
    readonly_fields = ('post_year_str', 'raw', 'fetch_time',
                        'time_created', 'time_modified',)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0006_auto_20150723_1322'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='account',
            options={'ordering': ['username']},
        ),
        migrations.AlterModelOptions(
            name='bookmark',
            options={'ordering': ['-post_time']},
        ),
        migrations.AlterField(
            model_name='account',
            name='time_created',
            field=models.DateTimeField(help_text=b'The time this item was created in the database.', auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='time_modified',
            field=models.DateTimeField(help_text=b'The time this item was last saved to the database.', auto_now=True),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='fetch_time',
            field=models.DateTimeField(help_text=b"The time the item's data was last fetched, and was new or changed.", null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='is_private',
            field=models.BooleanField(default=False, help_text=b'If set, this item will not be shown on public-facing pages.'),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='time_created',
            field=models.DateTimeField(help_text=b'The time this item was created in the database.', auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='time_modified',
            field=models.DateTimeField(help_text=b'The time this item was last saved to the database.', auto_now=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0017_account_is_active'),
    ]

    operations = [
        migrations.AddField(
            model_name='tweet',
            name='text_html',
            field=models.TextField(blank=True, help_text="An HTMLified version of the Tweet's text"),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0005_auto_20150730_1350'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='account',
            options={'ordering': ['-time_created']},
        ),
        migrations.RemoveField(
            model_name='account',
            name='screen_name',
        ),
    ]



import calendar
import datetime
import pytz

from django import template
from django.conf import settings
from django.db.models import Count
from django.utils.html import format_html

from ..models import Account, Album, Artist, Scrobble, Track
from ...core.utils import get_annual_item_counts


register = template.Library()


def check_top_kwargs(**kwargs):
    """
    Used to check the supplied kwargs for top_albums(), top_artist() and
    top_tracks().
    """

    account = kwargs['account']
    limit   = kwargs['limit']
    date    = kwargs['date']
    period  = kwargs['period']

    if account is not None and not isinstance(account, Account):
        raise TypeError('`account` must be an Account instance, '
                        'not a %s' % type(account))

    if limit != 'all' and isinstance(limit, int) == False:
        raise ValueError("`limit` must be an integer or 'all'")

    if date is not None and not isinstance(date, datetime.datetime) \
            and not isinstance(date, datetime.date):
        raise TypeError('`date` must be a datetime or date, '
                        'not a %s' % type(date))

    if period not in ['day', 'week', 'month', 'year']:
        raise TypeError(
                '`period` must be one of "day", "week", "month" or "year", '
                'not %s' % type(period))


def get_period_times(date, period):
    """
    Makes the min_post_time and max_post_time for restricting top_albums(),
    top_artists() or top_tracks() to a particular time period.

    Arguments:
    date -- A datetime or date.
    period -- String, 'day', 'week', 'month' or 'year'.
    """
    # First create start/end datetimes with the correct times:
    if isinstance(date, datetime.datetime):
        min_time = date.replace(hour=0, minute=0, second=0)
        max_time = date.replace(
                            hour=23, minute=59, second=59, microsecond=999999)
    else:
        # `date` is a datetime.date
        min_time = datetime.datetime.combine(
                    date, datetime.datetime.min.time()).replace(tzinfo=pytz.utc)
        max_time = datetime.datetime.combine(
                    date, datetime.datetime.max.time()).replace(tzinfo=pytz.utc)

    if period == 'week':
        # Default, Monday:
        start_day = 0 

        if hasattr(settings, 'DITTO_WEEK_START'):
            if isinstance(settings.DITTO_WEEK_START, int) \
                and settings.DITTO_WEEK_START >= 0 \
                and settings.DITTO_WEEK_START <= 6:
                start_day = settings.DITTO_WEEK_START
            else:
                raise ValueError(
                    'The DITTO_WEEK_START setting should be an int from 0 to 6')

        # Which day is `date` on?
        day_of_week = min_time.weekday()
        start_offset = datetime.timedelta(start_day - day_of_week)

        if start_day > day_of_week:
            # e.g. Week starts on a Sunday, so in previous week.
            start_offset -= datetime.timedelta(weeks=1)

        min_time = min_time + start_offset
        max_time = min_time + datetime.timedelta(weeks=1) \
                                      - datetime.timedelta(microseconds=1)
        
    elif period == 'month':
        min_time = min_time.replace(day=1)
        # Last day of month:
        end_day = calendar.monthrange(
                                max_time.year, max_time.month)[1]
        max_time = max_time.replace(day=end_day)

    elif period == 'year':
        min_time = min_time.replace(month=1, day=1)
        max_time = max_time.replace(month=12, day=31)

    return min_time, max_time


@register.simple_tag
def top_albums(account=None, artist=None, limit=10, date=None, period='day'):
    """Returns a QuerySet of most-scrobbled Albums, with the most-scrobbled
    first.

    Restrict to Albums by one Artist by suppling the `artist`.
    Restrict to only one user's scrobbles by supplying the `account`.

    By default gets all Albums.
    Restrict to a day, month or year by supplying a `date` within that
    day/week/month/year AND the `period` of 'day', 'week', 'month' or 'year'.

    Keyword arguments:
    account -- An Account object or None (for Scrobbles by all Accounts).
    artist -- An Artist object or None.
    limit -- Maximum number to fetch. Default is 10. 'all' for all Albums.
    date -- A datetime or date, for getting Albums from a single time period.
    period -- A String: 'day', 'week', 'month', or 'year'.
    """

    check_top_kwargs(**{
        'account': account,
        'limit': limit,
        'date': date,
        'period': period,
    })

    if artist is not None and not isinstance(artist, Artist):
        raise TypeError('artist must be an Artist instance, '
                        'not a %s' % type(artist))

    qs_kwargs = {}

    if account:
        qs_kwargs['account'] = account

    if artist:
        qs_kwargs['artist'] = artist

    if date and period:
        min_post_time, max_post_time = get_period_times(date, period)
        qs_kwargs['min_post_time'] = min_post_time
        qs_kwargs['max_post_time'] = max_post_time

    qs = Album.objects.with_scrobble_counts(**qs_kwargs)

    if limit != 'all':
        qs = qs[:limit]

    return qs


@register.simple_tag
def top_artists(account=None, limit=10, date=None, period='day'):
    """Returns a QuerySet of the most-scrobbled Artists, with the
    most-scrobbled first.

    Restrict to only one user's scrobbles by supplying the `account`.

    By default gets all Artists.
    Restrict to a day, month or year by supplying a `date` within that
    day/week/month/year AND the `period` of 'day', 'week', 'month' or 'year'.

    Keyword arguments:
    account -- An Account object or None (for Scrobbles by all Accounts).
    limit -- Maximum number to fetch. Default is 10. 'all' for all Artists.
    date -- A datetime or date, for getting Artists from a single time period.
    period -- A String: 'day', 'week', 'month', or 'year'.
    """
    check_top_kwargs(**{
        'account': account,
        'limit': limit,
        'date': date,
        'period': period,
    })

    qs_kwargs = {}

    if account:
        qs_kwargs['account'] = account

    if date and period:
        min_post_time, max_post_time = get_period_times(date, period)
        qs_kwargs['min_post_time'] = min_post_time
        qs_kwargs['max_post_time'] = max_post_time

    qs = Artist.objects.with_scrobble_counts(**qs_kwargs)

    if limit != 'all':
        qs = qs[:limit]

    return qs


@register.simple_tag
def top_tracks(account=None, album=None, artist=None, limit=10, date=None, period='day'):
    """
    Returns a QuerySet of most-scrobbled Tracks, with the most-scrobbled
    first.

    Restrict to Tracks from one Album by supplying the 'album'.
    Restrict to Tracks by one Artist by suppling the `artist`.
    Restrict to only one user's scrobbles by supplying the `account`.

    By default gets all Tracks.
    Restrict to a day, month or year by supplying a `date` within that
    day/week/month/year AND the `period` of 'day', 'week', 'month' or 'year'.

    Keyword arguments:
    account -- An Account object or None (for Scrobbles by all Accounts).
    album -- An Album object or None.
    artist -- An Artist object or None.
    limit -- Maximum number to fetch. Default is 10. 'all' for all Tracks.
    date -- A datetime or date, for getting Tracks from a single time period.
    period -- A String: 'day', 'week', 'month', or 'year'.
    """

    check_top_kwargs(**{
        'account': account,
        'limit': limit,
        'date': date,
        'period': period,
    })

    if album is not None and type(album) is not Album:
        raise TypeError('album must be an Album instance, '
                        'not a %s' % type(album))

    if artist is not None and type(artist) is not Artist:
        raise TypeError('artist must be an Artist instance, '
                        'not a %s' % type(artist))

    qs_kwargs = {}

    if account:
        qs_kwargs['account'] = account

    if album:
        qs_kwargs['album'] = album

    if artist:
        qs_kwargs['artist'] = artist

    if date and period:
        min_post_time, max_post_time = get_period_times(date, period)
        qs_kwargs['min_post_time'] = min_post_time
        qs_kwargs['max_post_time'] = max_post_time

    qs = Track.objects.with_scrobble_counts(**qs_kwargs)

    if limit != 'all':
        qs = qs[:limit]

    return qs


@register.simple_tag
def recent_scrobbles(account=None, limit=10):
    """Returns a QuerySet of the most recent Scrobbles by all Accounts, or one,
    most recent first.

    Keyword arguments:
    account -- An Account object or None (for Scrobbles by all Accounts).
    limit -- Maximum number to fetch. Default is 10.
    """
    if account is not None and not isinstance(account, Account):
        raise TypeError('account must be an Account instance, '
                        'not a %s' % type(account))

    if isinstance(limit, int) == False:
        raise ValueError("`limit` must be an integer")

    if type(account) is Account:
        return account.get_recent_scrobbles(limit)
    else:
        return Scrobble.objects.all()\
                            .order_by('-post_time')\
                            .prefetch_related('artist', 'track')[:limit]


@register.simple_tag
def day_scrobbles(date, account=None):
    """
    Returns a QuerySet of all Scrobbles from a particular day, in ascending
    order.

    Restrict to only one user's scrobbles by supplying the `account`.

    Keyword arguments:
    date -- A datetime or date. Required.
            If a datetime, we use the start and end of this day.
    account -- An Account object or None (default, Scrobbles by all Accounts).
    """
    if not isinstance(date, datetime.datetime) and not isinstance(
                                                        date, datetime.date):
        raise TypeError('date must be a datetime or date, '
                        'not a %s' % type(date))

    if account is not None and not isinstance(account, Account):
        raise TypeError('account must be an Account instance, '
                        'not a %s' % type(account))

    qs_kwargs = {}

    if isinstance(date, datetime.datetime):
        qs_kwargs['post_time__gte'] = date.replace(
                                hour=0, minute=0, second=0, microsecond=0)
        qs_kwargs['post_time__lte'] = date.replace(
                        hour=23, minute=59, second=59, microsecond=999999)
    else:
        # `date` is a datetime.date
        # __date filter is only available from Django >= 1.9
        qs_kwargs['post_time__contains'] = date

    qs = Scrobble.objects

    if account:
        qs_kwargs['account'] = account

    return qs.filter(**qs_kwargs).prefetch_related('artist', 'track')\
                                    .order_by('post_time')


@register.simple_tag
def annual_scrobble_counts(account=None):
    """
    Get the number of Scrobbles per year.
    Returns a list of dicts, sorted by year, like:
        [ {'year': 2015, 'count': 1234}, {'year': 2016, 'count': 9876} ]

    Keyword arguments:
    account -- An Account object or None (for Scrobbles by all Accounts).
    """

    if account is not None and not isinstance(account, Account):
        raise TypeError('account must be an Account instance, '
                        'not a %s' % type(account))

    qs = Scrobble.objects

    if account:
        qs = qs.filter(account=account)

    return get_annual_item_counts(qs)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0009_auto_20150729_1056'),
    ]

    operations = [
        migrations.AddField(
            model_name='bookmark',
            name='latitude',
            field=models.DecimalField(null=True, max_digits=12, decimal_places=9, blank=True),
        ),
        migrations.AddField(
            model_name='bookmark',
            name='longitude',
            field=models.DecimalField(null=True, max_digits=12, decimal_places=9, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0005_auto_20150626_1702'),
    ]

    operations = [
        migrations.AlterField(
            model_name='account',
            name='url',
            field=models.URLField(help_text=b"eg, 'https://pinboard.in/u:philgyford'", unique=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='summary',
            field=models.CharField(help_text=b"eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc. No HTML.", max_length=255, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0013_auto_20151026_1546'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='bookmarktag',
            options={'verbose_name_plural': 'Tags', 'verbose_name': 'Tag'},
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='is_private',
            field=models.BooleanField(default=False, help_text='If true, this item will not be shown on public-facing pages.'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
import ditto.core.models
import taggit.managers


class Migration(migrations.Migration):

    dependencies = [
        ('taggit', '0002_auto_20150616_2121'),
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, primary_key=True, auto_created=True)),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('api_key', models.CharField(max_length=255, blank=True, verbose_name='API Key')),
                ('api_secret', models.CharField(max_length=255, blank=True, verbose_name='API Secret')),
                ('is_active', models.BooleanField(default=True, help_text="If false, new Photos won't be fetched.")),
            ],
            options={
                'ordering': ['user__realname'],
            },
        ),
        migrations.CreateModel(
            name='Photo',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, primary_key=True, auto_created=True)),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('title', models.CharField(max_length=255, blank=True)),
                ('permalink', models.URLField(blank=True, help_text="URL of the item on the service's website.")),
                ('summary', models.CharField(max_length=255, blank=True, help_text="eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc. No HTML.")),
                ('is_private', models.BooleanField(default=False, help_text='If true, this item will not be shown on public-facing pages.')),
                ('fetch_time', models.DateTimeField(null=True, blank=True, help_text="The time the item's data was last fetched, and was new or changed.")),
                ('post_time', models.DateTimeField(null=True, blank=True, help_text='The time the item was originally posted/created on its service.')),
                ('latitude', models.DecimalField(null=True, blank=True, decimal_places=9, max_digits=12)),
                ('longitude', models.DecimalField(null=True, blank=True, decimal_places=9, max_digits=12)),
                ('raw', models.TextField(blank=True, help_text='eg, the raw JSON from the API.')),
                ('flickr_id', models.BigIntegerField(help_text='ID of this photo on Flickr.', db_index=True, unique=True)),
                ('description', models.TextField(blank=True, help_text='Can contain HTML')),
                ('secret', models.CharField(max_length=20)),
                ('original_secret', models.CharField(max_length=20)),
                ('server', models.CharField(max_length=20)),
                ('farm', models.PositiveSmallIntegerField()),
                ('license', models.CharField(max_length=50, choices=[('0', 'All Rights Reserved'), ('1', 'Attribution-NonCommercial-ShareAlike License'), ('2', 'Attribution-NonCommercial License'), ('3', 'Attribution-NonCommercial-NoDerivs License'), ('4', 'Attribution License'), ('5', 'Attribution-ShareAlike License'), ('6', 'Attribution-NoDerivs License'), ('7', 'No known copyright restrictions'), ('8', 'United States Government Work')])),
                ('rotation', models.PositiveSmallIntegerField(help_text='Current clockwise rotation, in degrees, by which the smaller image sizes differ from the original image.', default=0)),
                ('original_format', models.CharField(max_length=10, help_text="eg, 'png'")),
                ('safety_level', models.PositiveSmallIntegerField(choices=[(0, 'none'), (1, 'Safe'), (2, 'Moderate'), (3, 'Restricted')], default=0)),
                ('has_people', models.BooleanField(default=False, help_text='Are there Flickr users in this photo?')),
                ('last_update_time', models.DateTimeField(null=True, blank=True, help_text='The last time the photo, or any of its metadata (tags, comments, etc.) was modified on Flickr. UTC.')),
                ('taken_time', models.DateTimeField(null=True, blank=True, help_text="In the Flickr user's timezone.")),
                ('taken_granularity', models.PositiveSmallIntegerField(choices=[(0, 'Y-m-d H:i:s'), (4, 'Y-m'), (6, 'Y'), (8, 'Circa...')], default=0)),
                ('taken_unknown', models.BooleanField(default=False)),
                ('view_count', models.PositiveIntegerField(help_text='How many times this had been viewed when fetched', default=0)),
                ('comment_count', models.PositiveIntegerField(help_text='How many comments this had been when fetched', default=0)),
                ('media', models.CharField(max_length=10, choices=[('photo', 'Photo'), ('video', 'Video')], default='photo')),
                ('sizes_raw', models.TextField(blank=True, help_text='eg, the raw JSON from the API - flickr.photos.getSizes.')),
                ('width_t', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Thumbnail width')),
                ('height_t', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Thumbnail height')),
                ('width_m', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Small width')),
                ('height_m', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Small height')),
                ('width_n', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Small 320 width')),
                ('height_n', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Small 320 height')),
                ('width', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Medium width')),
                ('height', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Medium height')),
                ('width_z', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Medium 640 width')),
                ('height_z', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Medium 640 height')),
                ('width_c', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Medium 800 width')),
                ('height_c', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Medium 800 height')),
                ('width_b', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Large width')),
                ('height_b', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Large height')),
                ('width_h', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Large 1600 width')),
                ('height_h', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Large 1600 height')),
                ('width_k', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Large 2048 width')),
                ('height_k', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Large 2048 height')),
                ('width_o', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Original width')),
                ('height_o', models.PositiveSmallIntegerField(null=True, blank=True, verbose_name='Original height')),
                ('geo_is_private', models.BooleanField(default=False, help_text="If true, the Photo's location info should not be displayed.")),
                ('location_accuracy', models.PositiveSmallIntegerField(null=True, blank=True, default=1, help_text='World is 1; Country is ~3; Region is ~6; City is ~11; Street is ~16.')),
                ('location_context', models.PositiveSmallIntegerField(choices=[(0, 'not defined'), (1, 'indoors'), (2, 'outdoors')], default=0)),
                ('location_place_id', models.CharField(max_length=30, blank=True)),
                ('location_woeid', models.CharField(max_length=30, blank=True)),
                ('locality_name', models.CharField(max_length=255, blank=True)),
                ('locality_place_id', models.CharField(max_length=30, blank=True)),
                ('locality_woeid', models.CharField(max_length=30, blank=True)),
                ('county_name', models.CharField(max_length=255, blank=True)),
                ('county_place_id', models.CharField(max_length=30, blank=True)),
                ('county_woeid', models.CharField(max_length=30, blank=True)),
                ('region_name', models.CharField(max_length=255, blank=True)),
                ('region_place_id', models.CharField(max_length=30, blank=True)),
                ('region_woeid', models.CharField(max_length=30, blank=True)),
                ('country_name', models.CharField(max_length=255, blank=True)),
                ('country_place_id', models.CharField(max_length=30, blank=True)),
                ('country_woeid', models.CharField(max_length=30, blank=True)),
                ('exif_raw', models.TextField(blank=True, help_text='The raw JSON from the API from flickr.photos.getExif.')),
                ('exif_camera', models.CharField(max_length=50, blank=True)),
                ('exif_lens_model', models.CharField(max_length=50, blank=True, help_text="eg, 'E PZ 16-50mm F3.5-5.6 OSS'.")),
                ('exif_aperture', models.CharField(max_length=30, blank=True, help_text="eg, 'f/13.0'.")),
                ('exif_exposure', models.CharField(max_length=30, blank=True, help_text="eg, '0.01 sec (1/100)'.")),
                ('exif_flash', models.CharField(max_length=30, blank=True, help_text="eg, 'Off, Did not fire'.")),
                ('exif_focal_length', models.CharField(max_length=10, blank=True, help_text="eg, '38 mm.'")),
                ('exif_iso', models.IntegerField(blank=True, help_text="eg, '100'.")),
            ],
            options={
                'ordering': ('-taken_time',),
            },
            bases=(ditto.core.models.DiffModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='TaggedPhoto',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, primary_key=True, auto_created=True)),
                ('flickr_id', models.CharField(max_length=200, help_text="The tag's ID on Flickr", verbose_name='Flickr ID')),
                ('machine_tag', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'Photo/Tag Relationship',
            },
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, primary_key=True, auto_created=True)),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('nsid', models.CharField(max_length=50, verbose_name='NSID', unique=True)),
                ('is_pro', models.BooleanField(verbose_name='Is Pro?', default=False)),
                ('iconserver', models.CharField(max_length=20)),
                ('iconfarm', models.PositiveIntegerField()),
                ('username', models.CharField(max_length=50, unique=True)),
                ('realname', models.CharField(max_length=255)),
                ('location', models.CharField(max_length=255, blank=True)),
                ('description', models.TextField(blank=True, help_text='May contain HTML')),
                ('photos_url', models.URLField(max_length=255, verbose_name='Photos URL')),
                ('profile_url', models.URLField(max_length=255, verbose_name='Profile URL')),
                ('photos_count', models.PositiveIntegerField(default=0)),
                ('photos_views', models.PositiveIntegerField(default=0)),
                ('photos_first_date', models.DateTimeField(null=True)),
                ('photos_first_date_taken', models.DateTimeField(null=True)),
                ('fetch_time', models.DateTimeField(null=True, blank=True, help_text='The time the data was last fetched, and was new or changed.')),
                ('raw', models.TextField(blank=True, help_text='eg, the raw JSON from the API.')),
                ('timezone_id', models.CharField(max_length=50, help_text="eg, 'Europe/London'.")),
            ],
            options={
                'ordering': ['realname'],
            },
            bases=(ditto.core.models.DiffModelMixin, models.Model),
        ),
        migrations.AddField(
            model_name='taggedphoto',
            name='author',
            field=models.ForeignKey(to='flickr.User', on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='taggedphoto',
            name='content_object',
            field=models.ForeignKey(to='flickr.Photo', related_name='flickr_taggedphoto_items', on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='taggedphoto',
            name='tag',
            field=models.ForeignKey(to='taggit.Tag', related_name='flickr_taggedphoto_items',  on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='photo',
            name='tags',
            field=taggit.managers.TaggableManager(blank=True, to='taggit.Tag', verbose_name='Tags', help_text='A comma-separated list of tags.', through='flickr.TaggedPhoto'),
        ),
        migrations.AddField(
            model_name='photo',
            name='user',
            field=models.ForeignKey(to='flickr.User',  on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='account',
            name='user',
            field=models.ForeignKey(null=True, blank=True, to='flickr.User', on_delete=django.db.models.deletion.SET_NULL),
        ),
    ]



import datetime
import pytz

from django import template
from django.db.models import Count

from ..models import Bookmark
from ...core.utils import get_annual_item_counts


register = template.Library()

@register.simple_tag
def recent_bookmarks(account=None, limit=10):
    """Returns a QuerySet of recent public Bookmarks, in reverse-chronological
    order.

    Keyword arguments:
    account -- An account username, 'philgyford', or None to fetch for all.
    limit -- Maximum number to fetch. Default is 10.
    """
    bookmarks = Bookmark.public_objects.all()
    if account is not None:
        bookmarks = bookmarks.filter(account__username=account)
    bookmarks = bookmarks.prefetch_related('account')
    return bookmarks[:limit]

@register.simple_tag
def day_bookmarks(date, account=None):
    """Returns a QuerySet of public Bookmarks posted on a specific date.

    Arguments:
    date -- A date object.

    Keyword arguments:
    account -- An account username, 'philgyford', or None to fetch for all.
    """
    start = datetime.datetime.combine(date, datetime.time.min).replace(
                                                            tzinfo=pytz.utc)
    end   = datetime.datetime.combine(date, datetime.time.max).replace(
                                                            tzinfo=pytz.utc)
    bookmarks = Bookmark.public_objects.filter(post_time__range=[start, end])
    if account is not None:
        bookmarks = bookmarks.filter(account__username=account)
    bookmarks = bookmarks.prefetch_related('account')
    return bookmarks


@register.simple_tag
def annual_bookmark_counts(account=None):
    """
    Get the number of public Bookmarks per year.
    Returns a list of dicts, sorted by year, like:
        [ {'year': 2015, 'count': 1234}, {'year': 2016, 'count': 9876} ]

    Keyword arguments:
    account -- An account username, 'philgyford', or None to fetch for all.
    """
    bookmarks = Bookmark.public_objects

    if account:
        bookmarks = bookmarks.filter(account__username=account)

    return get_annual_item_counts(bookmarks)


@register.simple_tag
def popular_bookmark_tags(limit=10):
    return Bookmark.tags.most_common()[:limit]




from . import FetchCommand
from ...fetch.multifetchers import PhotosetsMultiAccountFetcher


class Command(FetchCommand):
    """Fetches photosets from Flickr

    For all accounts:
        ./manage.py fetch_flickr_photosets

    For one account:
        ./manage.py fetch_flickr_photosets --account=35034346050@N01
    """

    # What we're fetching:
    singular_noun = 'Photoset'
    plural_noun = 'Photosets'

    help = "Fetches all photosets for one or all Flickr Accounts (photo data must already be fetched)."

    def handle(self, *args, **options):
        # We might be fetching for a specific account or all (None).
        nsid = options['account'] if options['account'] else None;

        results = PhotosetsMultiAccountFetcher(nsid=nsid).fetch()

        self.output_results(results, options.get('verbosity', 1))



from django.http import Http404
from django.utils.translation import ugettext as _
from django.views.generic import DetailView, ListView
from django.views.generic.detail import SingleObjectMixin

from taggit.models import Tag

from ..core.views import PaginatedListView
from .models import Account, Photo, Photoset, TaggedPhoto, User


class PhotosOrderMixin(object):
    """
    For pages which list Photos and can change the order they're viewed in.
    Can have 'order' in the GET string, with values of 'uploaded' or 'taken'.

    Adds an 'order' key to the context_data, with value of 'uploaded'/'taken'.
    """

    def get_ordering(self):
        args = self.request.GET
        if 'order' in args and args['order'] == 'taken':
            return '-taken_time'
        else:
            return '-post_time'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['order'] = 'uploaded' if self.get_ordering() == '-post_time' else 'taken'
        return context

    def get_queryset(self):
        """Order by -taken_time or -post_time.
        If ordering by taken_time, exclude Photos where taken_unknown = True.
        """
        queryset = super().get_queryset()

        # Not sure why we need to repeat some of this from
        # ListView.get_queryset() here, but the UserDetail page, for one,
        # wasn't ordering by taken_time without this.
        ordering = self.get_ordering()
        if ordering:

            if ordering == '-taken_time':
                # Exclude where we don't know the taken time.
                queryset = queryset.filter(taken_unknown=False)

            import six
            if isinstance(ordering, six.string_types):
                ordering = (ordering,)
            queryset = queryset.order_by(*ordering)

        return queryset


class HomeView(PhotosOrderMixin, PaginatedListView):
    template_name = 'flickr/home.html'
    paginate_by = 48
    queryset = Photo.public_photo_objects

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['account_list'] = Account.objects.all()
        return context

    def get_queryset(self):
        """
        Adding the prefetch_related() to self.queryset caused some tests to
        fail for some reason.
        """
        queryset = super().get_queryset()
        return queryset.prefetch_related('user')



class PhotosetListView(ListView):
    template_name = 'flickr/photoset_list.html'
    queryset = Photoset.objects.all().prefetch_related('primary_photo', 'user')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['photoset_list'] = context['object_list']
        context['account_list'] = Account.objects.all()
        return context


class SingleUserMixin(SingleObjectMixin):
    """Used for views that need data about a User based on nsid in
    the URL, and its Account if it has one.
    """
    slug_field = 'nsid'
    slug_url_kwarg = 'nsid'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=User.objects.all())
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['flickr_user'] = self.object
        try:
            context['account'] = Account.objects.get(user=self.object)
        except Account.DoesNotExist:
            context['account'] = None
        return context


class UserDetailView(PhotosOrderMixin, SingleUserMixin, PaginatedListView):
    """A single Flickr User and its Photos.
    The user might have an Account associated with it, or might not.
    """
    template_name = 'flickr/user_detail.html'
    paginate_by = 48
    queryset = Photo.public_objects

    def get_queryset(self):
        "All public Photos from this Account."
        queryset = super().get_queryset()
        return queryset.filter(user=self.object).prefetch_related('user')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['photo_list'] = context['object_list']
        return context


class PhotoDetailView(DetailView):
    """Show a single Photo. It might be posted by one of the Accounts, or might
    be a Photo by someone else, favorited.
    """
    model = Photo
    slug_field = 'flickr_id'
    slug_url_kwarg = 'flickr_id'

    def get_object(self, queryset=None):
        """Do standard DetailView.get_object(), but return 404 if the Photo is
        private, OR if the URL's user NSID doesn't match the photo's."""
        obj = super().get_object(queryset)
        if obj.is_private or obj.user.nsid != self.kwargs['nsid']:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                                      {'verbose_name': obj._meta.verbose_name})
        return obj

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['flickr_user'] = context['photo'].user
        # We can show favorited Photos; they won't have an associated Account.
        try:
            context['account'] = Account.objects.get(
                                                user=context['flickr_user'])
        except Account.DoesNotExist:
            context['account'] = None
        return context


class TagListView(ListView):
    template_name = 'flickr/tag_list.html'
    context_object_name = 'tag_list'

    def get_queryset(self):
        return Photo.tags.most_common()[:100]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['account_list'] = Account.objects.all()
        return context


class TagDetailView(PhotosOrderMixin, SingleObjectMixin, PaginatedListView):
    "All Photos with a certain tag from all Accounts"
    template_name = 'flickr/tag_detail.html'
    allow_empty = False
    queryset = Photo.public_objects.prefetch_related('user')

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=Tag.objects.all())
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['tag'] = self.object
        context['account_list'] = Account.objects.all()
        context['photo_list'] = context['object_list']
        return context

    def get_queryset(self):
        """Show all the public Photos associated with this tag."""
        queryset = super().get_queryset()
        return queryset.filter(tags__slug__in=[self.object.slug])


class UserTagDetailView(PhotosOrderMixin, SingleUserMixin, PaginatedListView):
    "All Photos with a certain Tag from one User"
    template_name = 'flickr/user_tag_detail.html'
    allow_empty = False
    queryset = Photo.public_objects.prefetch_related('user')

    def get(self, request, *args, **kwargs):
        self.tag_object = self.get_tag_object()
        return super().get(request, *args, **kwargs)

    def get_tag_object(self):
        """Custom method for fetching the Tag."""
        try:
            obj = Tag.objects.get(slug=self.kwargs['tag_slug'])
        except Tag.DoesNotExist:
            raise Http404(_("No Tags found matching the query"))
        return obj

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['tag'] = self.tag_object
        context['photo_list'] = context['object_list']
        return context

    def get_queryset(self):
        """Show all the public Photos associated with this user."""
        queryset = super().get_queryset()
        return queryset.filter(user=self.object,
                                    tags__slug__in=[self.kwargs['tag_slug']])


class UserPhotosetListView(SingleUserMixin, ListView):
    template_name = 'flickr/user_photoset_list.html'
    queryset = Photoset.objects.all().prefetch_related('primary_photo', 'user')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['photoset_list'] = context['object_list']
        return context

    def get_queryset(self):
        """Show all Photosets associated with this user."""
        queryset = super().get_queryset()
        return queryset.filter(user=self.object)


class PhotosetDetailView(PhotosOrderMixin, SingleUserMixin, PaginatedListView):
    template_name = 'flickr/photoset_detail.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        # NOTE: photoset.objects.all() will return PRIVATE photos too.
        # You probably don't want to do that.
        context['photoset'] = self.photoset_object
        context['photo_list'] = context['object_list']
        return context

    def get_queryset(self):
        """Show all the public Photos in this Photoset."""
        self.photoset_object = self.get_photoset_object()
        return self.photoset_object.public_photos()

    def get_photoset_object(self):
        """Custom method for fetching the Photoset."""
        try:
            obj = Photoset.objects.get(user=self.object,
                                        flickr_id=self.kwargs['flickr_id'])
        except Photoset.DoesNotExist:
            raise Http404(_("No Photosets found matching the query"))
        return obj




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='photo',
            name='exif_iso',
            field=models.IntegerField(null=True, help_text="eg, '100'.", blank=True),
        ),
    ]



from django.conf.urls import url

from . import views


app_name = 'ditto'

urlpatterns = [
    url(
        regex=r"^$",
        view=views.HomeView.as_view(),
        name='home'
    ),
    #url(
        #regex=r"^tags$",
        #view=views.TagListView.as_view(),
        #name='tag_list'
    #),
    #url(
        #regex=r"^tags/(?P<slug>[^/]+)$",
        #view=views.TagDetailView.as_view(),
        #name='tag_detail'
    #),
    url(
        # /2016/04/18/twitter/favorites
        regex=r"^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})(?:/(?P<app>[a-z]+))?(?:/(?P<variety>[a-z\/]+|))?$",
        view=views.DayArchiveView.as_view(),
        name='day_archive'
    ),
]




# coding: utf-8
from django.apps import apps, AppConfig


class DittoCoreConfig(AppConfig):
    label = 'ditto_core'
    name = 'ditto.core'
    verbose_name = "Ditto Core"


class Apps(object):
    """Methods for seeing which Ditto apps are installed/enabled.
    At the moment installed is the same as enabled, but in future we may add
    conditions that mean an installed app can be disabled.

    So use installed to check if the code is physically in INSTALLED_APPS.
    And use enabled to check if we're allowed to use that app on the site.
    """

    def all(self):
        "A list of all possible Ditto apps that could be installed/enabled."
        return [
                'flickr',
                'lastfm',
                'pinboard',
                'twitter',
               ]

    def installed(self):
        "A list of all the installed Ditto apps."
        return [app for app in self.all() if self.is_installed(app)]

    def enabled(self):
        "A list of all the enabled Ditto apps."
        return [app for app in self.all() if self.is_enabled(app)]

    def is_installed(self, app_name):
        "Is this Ditto app installed?"
        return apps.is_installed('ditto.%s' % app_name)

    def is_enabled(self, app_name):
        """Determine if a particular Ditto app is installed and enabled.

        app_name is like 'pinboard' or 'twitter'.

        Usage:
            if is_enabled('pinboard'):
                print("Pinboard is enabled")

        Doesn't offer much over apps.is_installed() yet, but would let us add other
        conditions in future, like being able to enable/disable installed apps.
        """
        return apps.is_installed('ditto.%s' % app_name)


ditto_apps = Apps()




import codecs
import os
import re
import sys
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

read = lambda filepath: codecs.open(filepath, 'r', 'utf-8').read()

def get_entity(package, entity):
    """
    eg, get_entity('ditto', 'version') returns `__version__` value in
    `__init__.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    find = "__%s__ = ['\"]([^'\"]+)['\"]" % entity
    return re.search(find, init_py).group(1)

def get_version():
    return get_entity('ditto', 'version')

def get_license():
    return get_entity('ditto', 'license')

def get_author():
    return get_entity('ditto', 'author')

def get_author_email():
    return get_entity('ditto', 'author_email')

# Do `python setup.py tag` to tag with the current version number.
if sys.argv[-1] == 'tag':
    os.system("git tag -a %s -m 'version %s'" % (get_version(), get_version()))
    os.system("git push --tags")
    sys.exit()

# Do `python setup.py publish` to send current version to PyPI.
if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist")
    os.system("twine upload --config-file=.pypirc dist/*")
    sys.exit()

# Do `python setup.py testpublish` to send current version to Test PyPI.
# OUT OF DATE
if sys.argv[-1] == 'testpublish':
    os.system("python setup.py sdist upload -r pypitest")
    # os.system("python setup.py bdist_wheel upload")
    sys.exit()

setup(
    name='django-ditto',
    version=get_version(),
    packages=['ditto'],
    install_requires=[
        'django-imagekit>=4.0,<4.1',
        'django-sortedm2m>=1.5.0,<1.6',
        'django-taggit>=0.22.2,<0.30',
        'flickrapi>=2.3,<2.4',
        'pillow>=4.3.0,<4.4',
        'pytz',
        'twitter-text-python',
        'twython>=3.6.0,<3.7',
    ],
    dependency_links=[
    ],
    tests_require=[
        'factory-boy>=2.9.2,<3.0',
        'freezegun>=0.3.9,<0.4',
        'responses>=0.8.1,<0.9',
        'coverage',
    ],
    include_package_data=True,
    license=get_license(),
    description='A Django app to copy stuff from your accounts on Flickr, Last.fm, Pinboard and Twitter.',
    long_description=read(os.path.join(os.path.dirname(__file__), 'README.rst')),
    url='https://github.com/philgyford/django-ditto',
    author=get_author(),
    author_email=get_author_email(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    keywords='ditto twitter flickr pinboard last.fm',
)



from django.contrib import admin


class DittoItemModelAdmin(admin.ModelAdmin):

    def post_year_str(self, instance):
        "So Admin doesn't add a comma, like '2,016'."
        return str(instance.post_year)
    post_year_str.short_description = 'Post year'




import datetime
import pytz
import factory

from taggit import models as taggit_models

from . import models
from ..core.utils import datetime_now


class UserFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.User

    nsid = factory.Sequence(lambda n: '%s@N01' % (n * 10000))
    username = factory.Sequence(lambda n: 'user%04d' % n)
    realname = factory.Sequence(lambda n: 'User Name %04d' % n)
    iconserver = 1234
    iconfarm = 5
    timezone_id = 'America/Los_Angeles'
    photos_url = factory.Sequence(
                    lambda n: 'https://www.flickr.com/photos/user%04d/' % n)

    photos_first_date = factory.LazyAttribute(lambda o:
                               datetime_now() - datetime.timedelta(weeks=52)
                            )
    photos_first_date_taken = factory.LazyAttribute(lambda o:
                                datetime_now() - datetime.timedelta(weeks=52)
                            )
    fetch_time = datetime_now()

    avatar = factory.django.ImageField(filename='my_avatar.jpg')


class AccountFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Account

    user = factory.SubFactory(UserFactory,
            username=factory.Sequence(lambda n: n),
            realname=factory.Sequence(lambda n: 'User Name %04d' % n)
        )


class PhotoFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Photo

    user = factory.SubFactory(UserFactory,
            username=factory.Sequence(lambda n: n),
            realname=factory.Sequence(lambda n: 'User Name %04d' % n)
        )

    flickr_id = factory.Sequence(lambda n: (n * 1000000))
    title = factory.Sequence(lambda n: 'Photo %d' % n)
    secret = factory.Sequence(lambda n: (n * 10000))
    original_secret = factory.Sequence(lambda n: (n * 10001))
    original_format = 'jpg'
    server = '987'
    farm = 2
    media = 'photo'
    post_time = datetime_now()
    taken_time = factory.LazyAttribute(lambda o:
                                datetime_now() - datetime.timedelta(weeks=3)
                            )
    last_update_time = factory.LazyAttribute(lambda o:
                                datetime_now() - datetime.timedelta(weeks=2)
                            )
    fetch_time = factory.LazyAttribute(lambda o:
                                datetime_now() - datetime.timedelta(weeks=1)
                            )

    original_file = factory.django.ImageField(filename='example.jpg')
    video_original_file = factory.django.FileField(filename='example.mov')

    thumbnail_width = 100
    thumbnail_height = 67
    small_width = 240
    small_height = 160
    small_320_width = 320
    small_320_height = 213
    medium_width = 500
    medium_height = 333
    medium_640_width = 640
    medium_640_height = 427
    medium_800_width = 800
    medium_800_height = 533
    large_width = 1024
    large_height = 683
    large_1600_width = 1600
    large_1600_height = 1067
    large_2048_width = 2048
    large_2048_height = 1365
    original_width = 3000
    original_height = 2000


class TagFactory(factory.DjangoModelFactory):

    class Meta:
        model = taggit_models.Tag

    slug = factory.Sequence(lambda n: 'slug%d' % n)
    name = slug


class TaggedPhotoFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.TaggedPhoto

    tag = factory.SubFactory(TagFactory)
    flickr_id = factory.Sequence(lambda n: (n * 1000))
    author = factory.SubFactory(UserFactory)
    machine_tag = False
    content_object = factory.SubFactory(PhotoFactory)


class PhotosetFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Photoset

    user = factory.SubFactory(UserFactory,
            username=factory.Sequence(lambda n: n),
            realname=factory.Sequence(lambda n: 'User Name %04d' % n)
        )
    flickr_id = factory.Sequence(lambda n: (n * 1000000))
    title = factory.Sequence(lambda n: 'Photoset %d' % n)
    flickr_created_time = datetime_now()
    fetch_time = datetime_now()




# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-07-13 14:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0050_auto_20160713_1400'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='url',
            field=models.URLField(blank=True, default='', help_text='A URL provided by the user as part of their profile', max_length=255),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0015_auto_20150819_1349'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='account',
            options={'ordering': ['user__screen_name']},
        ),
        migrations.AlterField(
            model_name='account',
            name='consumer_key',
            field=models.CharField(help_text='(API Key)', blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='account',
            name='consumer_secret',
            field=models.CharField(help_text='(API Secret)', blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='account',
            name='last_favorite_id',
            field=models.BigIntegerField(help_text='The Twitter ID of the most recent favorited Tweet fetched.', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='last_recent_id',
            field=models.BigIntegerField(help_text='The Twitter ID of the most recent Tweet fetched.', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='time_created',
            field=models.DateTimeField(help_text='The time this item was created in the database.', auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='time_modified',
            field=models.DateTimeField(help_text='The time this item was last saved to the database.', auto_now=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='created_at',
            field=models.DateTimeField(help_text='UTC time when this Tweet was created on Twitter'),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='favorite_count',
            field=models.PositiveIntegerField(help_text='Approximately how many times this had been favorited when fetched', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='fetch_time',
            field=models.DateTimeField(help_text="The time the item's data was last fetched, and was new or changed.", null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='in_reply_to_screen_name',
            field=models.CharField(help_text="Screen name of the original Tweet's author, if this is a reply", blank=True, max_length=20),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='in_reply_to_status_id',
            field=models.BigIntegerField(help_text='The ID of the Tweet replied to, if any', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='in_reply_to_user_id',
            field=models.BigIntegerField(help_text="ID of the original Tweet's author, if this is a reply", null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='is_private',
            field=models.BooleanField(help_text='If set, this item will not be shown on public-facing pages.', default=False),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='language',
            field=models.CharField(help_text="A BCP 47 language identifier, or 'und' if it couldn't be detected", default='und', max_length=20),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='permalink',
            field=models.URLField(help_text="URL of the item on the service's website.", blank=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='quoted_status_id',
            field=models.BigIntegerField(help_text='The ID of the Tweet quoted, if any', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='raw',
            field=models.TextField(help_text='eg, the raw JSON from the API.', blank=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='retweet_count',
            field=models.PositiveIntegerField(help_text='Number of times this had been retweeted when fetched', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='source',
            field=models.CharField(help_text='Utility used to post the Tweet', blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='summary',
            field=models.CharField(help_text="eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc. No HTML.", blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='time_created',
            field=models.DateTimeField(help_text='The time this item was created in the database.', auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='time_modified',
            field=models.DateTimeField(help_text='The time this item was last saved to the database.', auto_now=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='created_at',
            field=models.DateTimeField(help_text='UTC time when this account was created on Twitter'),
        ),
        migrations.AlterField(
            model_name='user',
            name='favorites_count',
            field=models.PositiveIntegerField(help_text='The number of tweets this user has favorited in the account’s lifetime', default=0),
        ),
        migrations.AlterField(
            model_name='user',
            name='fetch_time',
            field=models.DateTimeField(help_text='The time the data was last fetched, and was new or changed.', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='followers_count',
            field=models.PositiveIntegerField(help_text='The number of followers this account has', default=0),
        ),
        migrations.AlterField(
            model_name='user',
            name='friends_count',
            field=models.PositiveIntegerField(help_text='Tne number of users this account is following.', default=0),
        ),
        migrations.AlterField(
            model_name='user',
            name='is_private',
            field=models.BooleanField(help_text="True if this user is 'protected'", default=False),
        ),
        migrations.AlterField(
            model_name='user',
            name='listed_count',
            field=models.PositiveIntegerField(help_text='The number of public lists this user is a member of', default=0),
        ),
        migrations.AlterField(
            model_name='user',
            name='name',
            field=models.CharField(help_text="eg, 'Samuel Pepys'", max_length=30),
        ),
        migrations.AlterField(
            model_name='user',
            name='raw',
            field=models.TextField(help_text='eg, the raw JSON from the API.', blank=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='screen_name',
            field=models.CharField(help_text="Username, eg, 'samuelpepys'", max_length=20),
        ),
        migrations.AlterField(
            model_name='user',
            name='statuses_count',
            field=models.PositiveIntegerField(help_text='The number of tweets, including retweets, by this user', default=0),
        ),
        migrations.AlterField(
            model_name='user',
            name='time_created',
            field=models.DateTimeField(help_text='The time this item was created in the database.', auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='time_modified',
            field=models.DateTimeField(help_text='The time this item was last saved to the database.', auto_now=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='url',
            field=models.URLField(help_text='A URL provided by the user as part of their profile', blank=True, default=''),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-02 16:45
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import sortedm2m.fields


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0010_photo_photosets'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='photo',
            name='photosets',
        ),
        migrations.AddField(
            model_name='photoset',
            name='photos',
            field=sortedm2m.fields.SortedManyToManyField(help_text=None, related_name='photosets', to='flickr.Photo'),
        ),
        migrations.AlterField(
            model_name='photoset',
            name='primary_photo',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='primary_photosets', to='flickr.Photo'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0010_remove_tweet_text'),
    ]

    operations = [
        migrations.AddField(
            model_name='tweet',
            name='text',
            field=models.TextField(default='', max_length=140),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-05 11:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0034_user_description_html'),
    ]

    operations = [
        migrations.AlterField(
            model_name='media',
            name='media_type',
            field=models.CharField(choices=[('animated_gif', 'Animated GIF'), ('photo', 'Photo'), ('video', 'Video')], max_length=8),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-17 16:26
from __future__ import unicode_literals

from django.db import migrations, models


def set_post_year(apps, schema_editor):
    """
    Re-saves every scrobble, so that its auto-generated `post_year` can
    be set.
    """
    Scrobble = apps.get_model('lastfm', 'Scrobble')
    for row in Scrobble.objects.all():
        row.save()


class Migration(migrations.Migration):

    dependencies = [
        ('lastfm', '0005_auto_20161117_1622'),
    ]

    operations = [
        migrations.AddField(
            model_name='scrobble',
            name='post_year',
            field=models.PositiveSmallIntegerField(blank=True, db_index=True, help_text='Set automatically on save', null=True),
        ),
        migrations.RunPython(set_post_year, reverse_code=migrations.RunPython.noop),
    ]




from . import UpdateTwitterCommand
from ...fetch.fetchers import UsersFetcher


class Command(UpdateTwitterCommand):
    """Fetches data for all Twitter Users in the DB, updating their info.

    Specify an account to use its API credentials:
    ./manage.py update_twitter_users --account=philgyford
    """

    help = "Fetches the latest data about each Twitter user"

    singular_noun = 'User'
    plural_noun = 'Users'

    def fetch(self, screen_name):
        return UsersFetcher(screen_name=screen_name).fetch()




import os
import time

from django.core.files import File

from twython import Twython, TwythonError

from . import FetchError
from .savers import TweetSaver, UserSaver
from ..models import Media, Tweet, User
from ...core.utils import datetime_now
from ...core.utils.downloader import DownloadException, filedownloader


# Classes which fetch data from the Twitter API for a single Account.
# You're probably not going to call these classes directly - use the classes
# in fetch.fetchers.*


# CLASSES HERE:
#
# Fetch
#   FetchVerify
#   FetchLookup
#     FetchUsers
#     FetchTweets
#   FetchNewTweets
#     FetchTweetsRecent
#     FetchTweetsFavorite
# FetchFiles


class Fetch(object):
    """Parent class for children that will call the Twitter API to fetch data
    for a single Account.

    Children should define their own methods for:
        _call_api()
        _save_results()

    and optionally:
        _post_save()
        _post_fetch()

    Use it like:
        account = Account.objects.get(pk=1)
        fetcher = RecentTweetsFetcher(account)
        result = fetcher.fetch()
    """
    # Will be an Account object, passed into init()
    account = None

    # Will be the Twython object for calling the Twitter API.
    api = None

    # Will be the UTC datetime that we fetch the results.
    fetch_time = None

    # Will be the results fetched from the API via Twython.
    results = []

    # Will be a list of all the Users/Tweets/etc created/updated:
    objects = []

    # What we'll return for each account:
    return_value = {}

    # When fetching Tweets or Users this will be the total amount fetched.
    results_count = 0

    def __init__(self, account):
        self.account = account

    def fetch(self):
        self._reset()

        if self.account.user:
            self.return_value['account'] = self.account.user.screen_name
        elif self.account.pk:
            self.return_value['account'] = 'Account: %s' % str(self.account)
        else:
            self.return_value['account'] = 'Unsaved Account'

        if self.account.has_credentials():
            self.api = Twython(
                self.account.consumer_key, self.account.consumer_secret,
                self.account.access_token, self.account.access_token_secret)
            self._fetch_pages()
            self._post_fetch()
        else:
            self.return_value['success'] = False
            self.return_value['messages'] = ['Account has no API credentials']

        self.return_value['fetched'] = self.results_count

        return self.return_value

    def _reset(self):
        self.fetch_time = datetime_now()
        self.results = []
        self.objects = []
        self.return_value = {}
        self.results_count = 0

    def _fetch_pages(self):
        try:
            self._call_api()
        except TwythonError as e:
            self.return_value['success'] = False
            self.return_value['messages'] = ['Error when calling API: %s' % e]
        else:
            # If we've got to the last 'page' of tweet results, we'll receive
            # an empty list from the API.
            if (len(self.results) > 0):
                self._save_results()
                self._post_save()

            self.return_value['success'] = True
        return

    def _since_id(self):
        return None

    def _call_api(self):
        """Define in child classes.
        Should call self.api.a_function_name() and set self.results with the
        results.
        """
        raise FetchError("Children of the Fetch class should define their own _call_api() method.")

    def _save_results(self):
        """Define in child classes.
        Should go through self._results() and, probably, call
        TweetSaver().save_tweet() or UserSaver().save_user() for each one.
        """
        self.objects = []

    def _post_save(self):
        """Can optionally be defined in child classes.
        Do any extra things that need to be done after saving a page of data.
        """
        pass

    def _post_fetch(self):
        """Can optionally be defined in child classes.
        Do any extra things that need to be done after we've fetched all data.
        """
        pass


class FetchVerify(Fetch):
    """For verifying an Account's API credentials, but ALSO fetches the user
    data for that single Account.
    """

    def _call_api(self):
        """Sets self.results to data for a single Twitter User."""
        self.results = self.api.verify_credentials()

    def _post_save(self):
        "Adds the one User we fetched to the data we'll return from fetch()."
        self.return_value['user'] = self.objects[0]

    def _save_results(self):
        """Creates/updates the user data.
        In other sibling classes this would loop through results and save each
        in turn, but here we only have a single result.
        """
        user = UserSaver().save_user(self.results, self.fetch_time)
        self.objects = [user]


class FetchLookup(Fetch):
    """
    Parent class for classes that call lookup* queries on the Twitter API.
    eg, lookup_user or lookup_status.
    """

    # Maximum number of users/tweets to ask for per query, allowed by the API:
    fetch_per_query = 100

    # Will be all the IDs we have yet to fetch from the API:
    ids_remaining_to_fetch = []

    # Maxmum number of requests allowed per 15 minute window:
    max_requests = 60

    def fetch(self, ids=[]):
        """
        Keyword arguments:
        ids -- A list of Twitter user/tweet IDs to fetch. Optional. If not
        supplied, we fetch data for all Users/Tweets in the DB. Up to the
        maximum allowed in a reasonable window. At time of writing, the API
        allows 100 per query, and 60 queries per 15 minute window. So 6000
        ids would be the maximum.
        """
        self._set_initial_ids(ids)
        return super().fetch()

    def _set_initial_ids(self, ids):
        """ids is a list of Twitter User/Tweet IDs, or an empty list."""

        if len(ids) == 0:
            # What's the biggest number we're able to fetch:
            limit = self.fetch_per_query * self.max_requests
            # Get all the IDs, up to the limit, ordered by fetch_time, so
            # that we get the least-recently updated this time.
            ids = self.model.objects.values_list('twitter_id', flat=True).order_by('fetch_time')[:limit]

        self.ids_remaining_to_fetch = ids

    def _post_save(self):
        # Remove the IDs we just fetched from the list:
        self.ids_remaining_to_fetch = self.ids_remaining_to_fetch[
                                                        self.fetch_per_query:]

        self.results_count += len(self.results)

        if self._more_to_fetch():
            time.sleep(0.5)
            self._fetch_pages()

    def _more_to_fetch(self):
        if len(self.ids_remaining_to_fetch) > 0:
            return True
        else:
            return False

    def _ids_to_fetch_in_query(self):
        """
        If self.fetch_per_query is 100, this returns the next 100 ids from
        self.ids_remaining_to_fetch.
        """
        return self.ids_remaining_to_fetch[:self.fetch_per_query]


class FetchUsers(FetchLookup):
    """For fetching users.

    Supply fetch() with a list of Twitter user IDs, and corresponding Users
    will be created/updated in the DB. Or, if no IDs are supplied, fetch the
    least-recently-fetched Users.
    """

    model = User

    def _call_api(self):
        # Sometimes this worked fine with numeric IDs, other times Tweepy
        # didn't put them in the URL and they had to be strings. Odd.
        ids = [str(id) for id in self._ids_to_fetch_in_query()]
        self.results = self.api.lookup_user(
                            user_id=ids,
                            include_entities=True
                        )

    def _save_results(self):
        for user in self.results:
            user_obj = UserSaver().save_user(user, self.fetch_time)
            self.objects.append(user_obj)


class FetchTweets(FetchLookup):
    """For fetching specific Tweets.

    Supply fetch() with a list of Twitter Tweet IDs, and coresponding Tweets
    will be created/updated in the DB. Or, if no IDs are supplied, fetch the
    least-recently-fetched Tweets.
    """

    model = Tweet

    def _call_api(self):
        ids = [str(id) for id in self._ids_to_fetch_in_query()]
        self.results = self.api.lookup_status(
                            id=ids,
                            tweet_mode='extended',
                            include_entities=True,
                            trim_user=False,
                            map=False
                        )

    def _save_results(self):
        for tweet in self.results:
            tweet_obj = TweetSaver().save_tweet(tweet, self.fetch_time)
            self.objects.append(tweet_obj)


class FetchNewTweets(Fetch):
    """A parent class for those which fetch "new" Tweets. ie, fetching a
    quantity of the most recent Tweets/favorited Tweets. As opposed to fetching
    a specified selection of Tweets, regardless of when they were.
    """

    # When fetching 'new' Tweets, after a query, this will be set as the
    # max_id to use for the next query.
    max_id = None

    # When fetching 'new' Tweets, this will be set as the highest ID
    # fetched, so it can be used to set account.last_recent_id or
    # account.last_favorite_id when we're done.
    last_id = None

    remaining_to_fetch = 0

    # Will be 'new' or 'number':
    fetch_type = 'new'

    def fetch(self, count='new'):
        """
        Keyword arguments:
        count -- Either 'new' (get the Tweets since the last fetch) or a number
                to fetch that many, up to the API limit (3200 currently).
        """
        if count == 'new':
            self.fetch_type = 'new'
        else:
            self.fetch_type = 'number'
            self.remaining_to_fetch = count

        return super().fetch()

    def _post_save(self):
        "After saving a page of results, what to do..."

        if self.last_id is None:
            self.last_id = self.results[0]['id']

        # The max_id for the next 'page' of tweets:
        self.max_id = self.results[-1]['id'] - 1

        if self.fetch_type == 'number':
            self.remaining_to_fetch -= len(self.results)

        self.results_count += len(self.results)

        if self._more_to_fetch():
            time.sleep(0.5)
            self._fetch_pages()

    def _more_to_fetch(self):
        if self.fetch_type == 'new':
            if self._since_id() is None or self.max_id > self._since_id():
                return True
            else:
                return False
        elif self.fetch_type == 'number':
            if self.remaining_to_fetch > 0:
                return True
            else:
                return False
        return False

    def _tweets_to_fetch_in_query(self):
        "How many Tweets to fetch in the current API query."
        to_fetch = 200
        if self.fetch_type == 'number' and self.remaining_to_fetch < 200:
            to_fetch = self.remaining_to_fetch
        return to_fetch


class FetchTweetsRecent(FetchNewTweets):
    """For fetching recent tweets by a single Account."""

    def _since_id(self):
        if self.fetch_type == 'new':
            return self.account.last_recent_id
        else:
            return None

    def _call_api(self):
        """Sets self.results to be the timeline of tweets for this Account.
        If the account's `last_recent_id` is set, we fetch tweets from that ID
        onwards, up to 200.
        Otherwise we fetch the most recent 200.
        """
        # account.last_recent_id might be None, in which case it's not used in
        # the API call:
        self.results = self.api.get_user_timeline(
                                user_id=self.account.user.twitter_id,
                                include_rts=True,
                                count=self._tweets_to_fetch_in_query(),
                                max_id=self.max_id,
                                since_id=self._since_id())

    def _post_fetch(self):
        "Set last_recent_id of our Account to the most recent Tweet fetched."
        if self.last_id is not None:
            self.account.last_recent_id = self.last_id
            self.account.save()

    def _save_results(self):
        """Takes the list of tweet data from the API and creates or updates the
        Tweet objects and the posters' User objects.
        Adds each new Tweet object to self.objects.
        """
        for tweet in self.results:
            tw = TweetSaver().save_tweet(tweet, self.fetch_time)
            self.objects.append(tw)


class FetchTweetsFavorite(FetchNewTweets):
    """For fetching tweets favorited by a single Account."""

    def _since_id(self):
        if self.fetch_type == 'new':
            return self.account.last_favorite_id
        else:
            return None

    def _call_api(self):
        """Sets self.results to be recent tweets favorited by this Account.
        If the account has `last_favorite_id` set, all the favorites since
        that ID are fetched (up to 200).
        Otherwise, the most recent 200 are fetched.
        """
        # account.last_favorite_id might be None, in which case it's not used in
        # the API call:
        self.results = self.api.get_favorites(
                                user_id=self.account.user.twitter_id,
                                count=self._tweets_to_fetch_in_query(),
                                max_id=self.max_id,
                                since_id=self._since_id())

    def _post_fetch(self):
        """Set the last_favorite_id of our Account to the most recent Tweet we
        fetched.
        """
        if self.last_id is not None:
            self.account.last_favorite_id = self.last_id
            self.account.save()

    def _save_results(self):
        """Takes the list of tweet data from the API and creates or updates the
        Tweet objects and the posters' User objects.
        Adds each new Tweet object to self.objects.
        """
        for tweet in self.results:
            tw = TweetSaver().save_tweet(tweet, self.fetch_time)
            # Associate this tweet with the Account's user:
            self.account.user.favorites.add(tw)
            self.objects.append(tw)


class FetchFiles(object):
    """
    For fetching image files and Animated GIFs' MP4 files.

    Doesn't inherit from Fetch because it doesn't use the API or rely on having
    an Account object.

    Doesn't fetch MP4s (or other movie files) for videos because the URLs
    for MP4 video files (as opposed to MP4 Animated GIF files) will be
    discontinued from 2016-08-01.
    https://twittercommunity.com/t/retiring-mp4-video-output/66093/15?u=philgyford

    Use it like:
        fetcher = FilesFetcher()
        result = fetcher.fetch()
    or:
        result = fetcher.fetch(fetch_all=True)
    """

    # What we'll return for each account:
    return_value = {}

    # When fetching Tweets or Users this will be the total amount fetched.
    results_count = 0

    def fetch(self, fetch_all=False):
        """
        Download and save original images for all Media objects
        (or just those that don't already have them).

        fetch_all -- Boolean. Fetch ALL images, even if we've already
                        got them?
        """

        self._fetch_files(fetch_all)

        self.return_value['fetched'] = self.results_count

        return self.return_value

    def _fetch_files(self, fetch_all):
        """
        Download and save original images for all Media objects
        (or just those that don't already have them).

        fetch_all -- Boolean. Fetch ALL images, even if we've already
                        got them?
        """

        media = Media.objects.all()

        if not fetch_all:
            media = media.filter(image_file='')

        error_messages = []

        for media_obj in media:
            try:
                self._fetch_and_save_file(
                                    media_obj=media_obj, media_type='image')
                self.results_count += 1
            except FetchError as e:
                error_messages.append(str(e))

            if media_obj.media_type == 'animated_gif':
                try:
                    self._fetch_and_save_file(
                                        media_obj=media_obj, media_type='mp4')
                    self.results_count += 1
                except FetchError as e:
                    error_messages.append(str(e))

        if len(error_messages) > 0:
            self.return_value['success'] = False
            self.return_value['messages'] = error_messages
        else:
            self.return_value['success'] = True

    def _fetch_and_save_file(self, media_obj, media_type):
        """
        Downloads an image file and saves it to the Media object.

        Expects:
            media_obj -- A Media object.
            media_type -- String, either 'image' or 'mp4'.

        Raises FetchError if something goes wrong.
        """

        if media_type == 'mp4':
            url = media_obj.mp4_url
            acceptable_content_types = ['video/mp4',]
        elif media_type == 'image':
            url = media_obj.image_url
            acceptable_content_types = [
                        'image/jpeg', 'image/jpg', 'image/png', 'image/gif',]
        else:
            raise FetchError('media_type should be "image" or "mp4"')

        filepath = False
        try:
            # Saves the file to /tmp/:
            filepath = filedownloader.download(url, acceptable_content_types)
        except DownloadException as e:
            raise FetchError(e)

        if filepath:
            # Reopen file and save to the Media object:
            reopened_file = open(filepath, 'rb')
            django_file = File(reopened_file)

            if media_type == 'mp4':
                media_obj.mp4_file.save(
                                    os.path.basename(filepath), django_file)
            else:
                media_obj.image_file.save(
                                    os.path.basename(filepath), django_file)







import datetime
import pytz
import factory

from . import models
from ..core.utils import datetime_now


class AccountFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Account

    username = factory.Sequence(lambda n: 'user%d' % n)
    url = factory.LazyAttribute(lambda obj: 'https://pinboard.com/%s' %
                                                                obj.username)
    api_token = factory.LazyAttribute(lambda obj: '%s:123ABC' % obj.username)


class BookmarkFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Bookmark

    # DittoItem properties:
    title = factory.Sequence(lambda n: 'A Title %d' % n)
    is_private = False

    # Bookmark properties:
    account = factory.SubFactory(AccountFactory)
    url = factory.Sequence(lambda n: 'http://www.example.com/%d' % n)

    post_time = factory.LazyAttribute(lambda o:
                        datetime_now()
                        - datetime.timedelta(hours=1)
                    )
    description = factory.Sequence(lambda n: 'A description of %d' % n)
    to_read = False

    # Leave tags empty by default.





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0015_auto_20151110_1308'),
    ]

    operations = [
        migrations.AlterField(
            model_name='account',
            name='api_token',
            field=models.CharField(verbose_name='API Token', max_length=51, help_text='eg, "philgyford:1234567890ABCDEFGHIJ"'),
        ),
        migrations.AlterField(
            model_name='account',
            name='url',
            field=models.URLField(verbose_name='URL', unique=True, max_length=255, help_text="eg, 'https://pinboard.in/u:philgyford'"),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0021_video'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='photo',
            name='tweet',
        ),
        migrations.RemoveField(
            model_name='video',
            name='tweet',
        ),
        migrations.RemoveField(
            model_name='tweet',
            name='photos_count',
        ),
        migrations.AddField(
            model_name='tweet',
            name='media_count',
            field=models.PositiveSmallIntegerField(default=0, blank=True, help_text='Number of Photos/Videos attached to this Tweet'),
        ),
        migrations.DeleteModel(
            name='Photo',
        ),
        migrations.DeleteModel(
            name='Video',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0033_auto_20160421_1602'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='description_html',
            field=models.TextField(help_text="An HTMLified version of the User's description", blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0030_auto_20151119_1649'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tweet',
            name='fetch_time',
            field=models.DateTimeField(blank=True, help_text="The time the item's data was last fetched.", null=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='fetch_time',
            field=models.DateTimeField(blank=True, help_text='The time the data was last fetched.', null=True),
        ),
    ]



from django.contrib import admin
from django.db import models
from django.forms import TextInput

from ..core.admin import DittoItemModelAdmin
from .models import Account, Photo, Photoset, User


@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = ('user', 'has_credentials', 'is_active',
                    'time_created', 'time_modified',)

    fieldsets = (
        (None, {
            'fields': ('user', 'is_active', )
        }),
        ('API', {
            'fields': ('api_key', 'api_secret',),
            'description': 'Keys and secrets require creation of an app at <a href="https://www.flickr.com/services/apps/create/apply/">flickr.com/...</a>',
        }),
        ('Data', {
            'classes': ('collapse',),
            'fields': ('time_created', 'time_modified',)
        }),
    )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {'widget': TextInput(
            attrs={
                'class': 'vLargeTextField',
            }
        )},
    }

    readonly_fields = ('time_created', 'time_modified',)

    def has_credentials(self, obj):
        return obj.has_credentials()
    has_credentials.boolean = True


@admin.register(User)
class UserAdmin(admin.ModelAdmin):
    list_display = ('show_avatar', 'realname', 'username', 'fetch_time',)
    list_display_links = ('show_avatar', 'realname', )

    fieldsets = (
        (None, {
            'fields': ('realname', 'username', 'nsid',
                'is_pro', 'location', 'description',
                'photos_url', 'profile_url', 'avatar',
                'photos_count', 'photos_views',
                'photos_first_date', 'photos_first_date_taken', 'timezone_id',)
        }),
        ('Data', {
            'classes': ('collapse',),
            'fields': ('iconserver', 'iconfarm', 'raw',
                        'fetch_time', 'time_created', 'time_modified',)
        }),
    )
    readonly_fields = ('raw', 'fetch_time', 'time_created', 'time_modified',)
    search_fields = ('realname', 'username',)

    def show_avatar(self, instance):
        return '<img src="%s" width="24" height="24" />' % (instance.avatar_url)
    show_avatar.allow_tags = True
    show_avatar.short_description = ''


@admin.register(Photoset)
class PhotosetAdmin(admin.ModelAdmin):
    list_display = ('title', 'flickr_created_time', 'fetch_time',)
    list_display_links = ('title', )

    fieldsets = (
        (None, {
            'fields': ('user', 'title', 'description', 'primary_photo',
                'flickr_id',
                'photo_count', 'video_count', 'view_count', 'comment_count',
                'last_update_time', 'flickr_created_time', 'photos',)
        }),
        ('Data', {
            'classes': ('collapse',),
            'fields': ('raw', 'fetch_time', 'time_created', 'time_modified',)
        }),
    )
    readonly_fields = ('raw', 'fetch_time', 'time_created', 'time_modified',)
    search_fields = ('title', 'description',)
    raw_id_fields = ('primary_photo', 'photos',)


class TaggedPhotoInline(admin.TabularInline):
    model = Photo.tags.through
    raw_id_fields = ('tag',)


@admin.register(Photo)
class PhotoAdmin(DittoItemModelAdmin):
    list_display = ('title', 'show_thumb', 'post_time', 'taken_time', )
    list_display_links = ('title', 'show_thumb',)
    list_filter = ('post_time', 'fetch_time', )
    date_hierarchy = 'post_time'
    search_fields = ['title', 'description',]

    fieldsets = (
        (None, {
            'fields': ('show_image', 'user', 'title', 'description', 'summary',
                'permalink', 'is_private', 'flickr_id', 'media', 'license',
                'original_file', 'video_original_file', )
        }),
        ('Times', {
            'classes': ('collapse',),
            'fields': ('post_time', 'post_year_str',
                        'last_update_time',
                        'taken_time', 'taken_year_str', 'taken_granularity',
                        'taken_unknown', )
        }),
        ('Counts, secrets, server, etc', {
            'classes': ('collapse',),
            'fields': ('view_count', 'comment_count', 'secret', 'original_secret', 'server', 'farm',                 'rotation', 'original_format', 'safety_level', 'has_people')
        }),
        ('Sizes', {
            'classes': ('collapse',),
            'fields': (
                'thumbnail_width', 'thumbnail_height', 'small_width', 'small_height', 'small_320_width', 'small_320_height', 'medium_width', 'medium_height', 'medium_640_width', 'medium_640_height', 'medium_800_width', 'medium_800_height', 'large_width', 'large_height', 'large_1600_width', 'large_1600_height', 'large_2048_width', 'large_2048_height', 'original_width', 'original_height',
                'mobile_mp4_width', 'mobile_mp4_height',
                'site_mp4_width', 'site_mp4_height',
                'hd_mp4_width', 'hd_mp4_height',
                'video_original_width', 'video_original_height',
            )
        }),
        ('Location', {
            'classes': ('collapse',),
            'fields': ('geo_is_private', 'latitude', 'longitude', 'location_accuracy', 'location_context', 'location_place_id', 'location_woeid',
                'locality_name', 'locality_place_id', 'locality_woeid',
                'county_name', 'county_place_id', 'county_woeid',
                'region_name', 'region_place_id', 'region_woeid',
                'country_name', 'country_place_id', 'country_woeid', )
        }),
        ('EXIF', {
            'classes': ('collapse',),
            'fields': ('exif_camera', 'exif_lens_model', 'exif_aperture', 'exif_exposure', 'exif_flash', 'exif_focal_length', 'exif_iso', )
        }),
        ('Data', {
            'classes': ('collapse',),
            'fields': ('raw', 'sizes_raw', 'exif_raw', 'fetch_time', 'time_created', 'time_modified',)
        }),
    )
    radio_fields = {'media': admin.HORIZONTAL}
    readonly_fields = ('post_year_str', 'taken_year_str', 'show_image', 'raw',
                        'fetch_time', 'time_created', 'time_modified',
                        'sizes_raw', 'exif_raw', )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {'widget': TextInput(
            attrs={
                'class': 'vLargeTextField',
            }
        )},
    }
    inlines = [TaggedPhotoInline,]
    raw_id_fields = ('user',)

    def show_thumb(self, instance):
        return '<img src="%s" width="%s" height="%s" />' % (instance.thumbnail_url, instance.thumbnail_width, instance.thumbnail_height)
    show_thumb.allow_tags = True
    show_thumb.short_description = 'Thumbnail'

    def show_image(self, instance):
        return '<img src="%s" width="%s" height="%s" />' % (instance.small_url, instance.small_width, instance.small_height)
    show_image.allow_tags = True
    show_image.short_description = 'Small image'

    def taken_year_str(self, instance):
        "So Admin doesn't add a comma, like '2,016'."
        return str(instance.taken_year)
    taken_year_str.short_description = 'Taken year'




# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-07-13 11:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0018_auto_20160525_1011'),
    ]

    operations = [
        migrations.AlterField(
            model_name='photo',
            name='safety_level',
            field=models.PositiveSmallIntegerField(choices=[(0, 'None'), (1, 'Safe'), (2, 'Moderate'), (3, 'Restricted')], default=0),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='account',
            name='consumer_key',
            field=models.CharField(help_text=b'(API Key)', max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='consumer_secret',
            field=models.CharField(help_text=b'(API Secret)', max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='screen_name',
            field=models.CharField(help_text=b"eg, 'philgyford'", unique=True, max_length=20, blank=True),
        ),
    ]
